@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "gas.h"
 #include "pref.h"
+#include "gettext.h"
 #include <stdio.h>
 #include <string.h>
 
@@ -21,7 +22,7 @@ bool isobaric_counterdiffusion(struct gasmix oldgasmix, struct gasmix newgasmix,
 	return get_he(oldgasmix) > 0 && results->dN2 > 0 && results->dHe < 0 && get_he(oldgasmix) && results->dN2 > 0 && 5 * results->dN2 > -results->dHe;
 }
 
-static bool gasmix_is_invalid(struct gasmix mix)
+bool gasmix_is_invalid(struct gasmix mix)
 {
 	return mix.o2.permille < 0;
 }
@@ -95,6 +96,15 @@ fraction_t get_gas_component_fraction(struct gasmix mix, enum gas_component comp
 	}
 }
 
+// O2 pressure in mbar according to the steady state model for the PSCR
+// NB: Ambient pressure comes in bar!
+int pscr_o2(const double amb_pressure, struct gasmix mix) {
+	int o2 = get_o2(mix) * amb_pressure - (int)((1.0 - get_o2(mix) / 1000.0) * prefs.o2consumption / (prefs.bottomsac * prefs.pscr_ratio) * 1000000);
+	if (o2 < 0.0) // He's dead, Jim.
+		o2 = 0.0;
+	return o2;
+}
+
 /* fill_pressures(): Compute partial gas pressures in bar from gasmix and ambient pressures, possibly for OC or CCR, to be
  * extended to PSCT. This function does the calculations of gas pressures applicable to a single point on the dive profile.
  * The structure "pressures" is used to return calculated gas pressures to the calling software.
@@ -122,9 +132,7 @@ void fill_pressures(struct gas_pressures *pressures, const double amb_pressure,
 		}
 	} else {
 		if (divemode == PSCR) { /* The steady state approximation should be good enough */
-			pressures->o2 = get_o2(mix) / 1000.0 * amb_pressure - (1.0 - get_o2(mix) / 1000.0) * prefs.o2consumption / (prefs.bottomsac * prefs.pscr_ratio / 1000.0);
-			if (pressures->o2 < 0) // He's dead, Jim.
-				pressures->o2 = 0;
+			pressures->o2 = pscr_o2(amb_pressure, mix) / 1000.0;
 			if (get_o2(mix) != 1000) {
 				pressures->he = (amb_pressure - pressures->o2) * get_he(mix) / (1000.0 - get_o2(mix));
 				pressures->n2 = (amb_pressure - pressures->o2) * get_n2(mix) / (1000.0 - get_o2(mix));
@@ -139,3 +147,32 @@ void fill_pressures(struct gas_pressures *pressures, const double amb_pressure,
 		}
 	}
 }
+
+enum gastype gasmix_to_type(struct gasmix mix)
+{
+	if (gasmix_is_air(mix))
+		return GASTYPE_AIR;
+	if (mix.o2.permille >= 980)
+		return GASTYPE_OXYGEN;
+	if (mix.he.permille == 0)
+		return mix.o2.permille >= 230 ? GASTYPE_NITROX : GASTYPE_AIR;
+	if (mix.o2.permille <= 180)
+		return GASTYPE_HYPOXIC_TRIMIX;
+	return mix.o2.permille <= 230 ? GASTYPE_NORMOXIC_TRIMIX : GASTYPE_HYPEROXIC_TRIMIX;
+}
+
+static const char *gastype_names[] = {
+	QT_TRANSLATE_NOOP("gettextFromC", "Air"),
+	QT_TRANSLATE_NOOP("gettextFromC", "Nitrox"),
+	QT_TRANSLATE_NOOP("gettextFromC", "Hypoxic Trimix"),
+	QT_TRANSLATE_NOOP("gettextFromC", "Normoxic Trimix"),
+	QT_TRANSLATE_NOOP("gettextFromC", "Hyperoxic Trimix"),
+	QT_TRANSLATE_NOOP("gettextFromC", "Oxygen")
+};
+
+const char *gastype_name(enum gastype type)
+{
+	if (type < 0 || type >= GASTYPE_COUNT)
+		return "";
+	return translate("gettextFromC", gastype_names[type]);
+}
