@@ -13,7 +13,6 @@
 #include "planner.h"
 #include "subsurface-time.h"
 #include "gettextfromc.h"
-#include "applicationstate.h"
 #include "metadata.h"
 #include "exif.h"
 #include "file.h"
@@ -36,7 +35,9 @@
 #include <QFont>
 #include <QApplication>
 #include <QTextDocument>
+#include <QPainter>
 #include <QProgressDialog>	// TODO: remove with convertThumbnails()
+#include <QSvgRenderer>
 #include <cstdarg>
 #include <cstdint>
 #ifdef Q_OS_UNIX
@@ -290,19 +291,6 @@ bool gpsHasChanged(struct dive *dive, struct dive *master, const QString &gps_te
 }
 #endif
 
-QList<int> getDivesInTrip(dive_trip_t *trip)
-{
-	QList<int> ret;
-	int i;
-	struct dive *d;
-	for_each_dive (i, d) {
-		if (d->divetrip == trip) {
-			ret.push_back(get_divenr(d));
-		}
-	}
-	return ret;
-}
-
 static xmlDocPtr get_stylesheet_doc(const xmlChar *uri, xmlDictPtr, int, void *, xsltLoadType)
 {
 	QFile f(QLatin1String(":/xslt/") + (const char *)uri);
@@ -419,6 +407,8 @@ QString getUserAgent()
 	// replace all other ':' with ' ' so that this is easy to parse
 #ifdef SUBSURFACE_MOBILE
 	QString userAgent = QString("Subsurface-mobile:%1(%2):").arg(subsurface_mobile_version()).arg(subsurface_canonical_version());
+#elif SUBSURFACE_DOWNLOADER
+	QString userAgent = QString("Subsurface-downloader:%1:").arg(subsurface_canonical_version());
 #else
 	QString userAgent = QString("Subsurface:%1:").arg(subsurface_canonical_version());
 #endif
@@ -551,14 +541,19 @@ QString get_depth_string(depth_t depth, bool showunit, bool showdecimal)
 	return get_depth_string(depth.mm, showunit, showdecimal);
 }
 
-QString get_depth_unit()
+QString get_depth_unit(bool metric)
 {
-	if (prefs.units.length == units::METERS)
+	if (metric)
 		return gettextFromC::tr("m");
 	else
 		return gettextFromC::tr("ft");
 }
 
+QString get_depth_unit()
+{
+	return get_depth_unit(prefs.units.length == units::METERS);
+}
+
 QString get_weight_string(weight_t weight, bool showunit)
 {
 	QString str = weight_string(weight.grams);
@@ -570,14 +565,19 @@ QString get_weight_string(weight_t weight, bool showunit)
 	return str;
 }
 
-QString get_weight_unit()
+QString get_weight_unit(bool metric)
 {
-	if (prefs.units.weight == units::KG)
+	if (metric)
 		return gettextFromC::tr("kg");
 	else
 		return gettextFromC::tr("lbs");
 }
 
+QString get_weight_unit()
+{
+	return get_weight_unit(prefs.units.weight == units::KG);
+}
+
 QString get_temperature_string(temperature_t temp, bool showunit)
 {
 	if (temp.mkelvin == 0) {
@@ -591,12 +591,17 @@ QString get_temperature_string(temperature_t temp, bool showunit)
 	}
 }
 
+QString get_temp_unit(bool metric)
+{
+	if (metric)
+		return QStringLiteral("°C");
+	else
+		return QStringLiteral("°F");
+}
+
 QString get_temp_unit()
 {
-	if (prefs.units.temperature == units::CELSIUS)
-		return QString("°C");
-	else
-		return QString("°F");
+	return get_temp_unit(prefs.units.temperature == units::CELSIUS);
 }
 
 QString get_volume_string(int mliter, bool showunit)
@@ -612,11 +617,17 @@ QString get_volume_string(volume_t volume, bool showunit)
 	return get_volume_string(volume.mliter, showunit);
 }
 
+QString get_volume_unit(bool metric)
+{
+	if (metric)
+		return gettextFromC::tr("ℓ");
+	else
+		return gettextFromC::tr("cuft");
+}
+
 QString get_volume_unit()
 {
-	const char *unit;
-	(void) get_volume_units(0, NULL, &unit);
-	return QString(unit);
+	return get_volume_unit(prefs.units.volume == units::LITER);
 }
 
 QString get_pressure_string(pressure_t pressure, bool showunit)
@@ -1262,6 +1273,23 @@ QString get_gas_string(struct gasmix gas)
 	return result;
 }
 
+QStringList get_dive_gas_list(const struct dive *d)
+{
+	QStringList list;
+	for (int i = 0; i < d->cylinders.nr; i++) {
+		const cylinder_t *cyl = get_cylinder(d, i);
+		/* Check if we have the same gasmix two or more times
+		 * If yes return more verbose string */
+		int same_gas = same_gasmix_cylinder(cyl, i, d, true);
+		if (same_gas == -1)
+			list.push_back(get_gas_string(cyl->gasmix));
+		else
+			list.push_back(get_gas_string(cyl->gasmix) + QString(" (%1 %2 ").arg(gettextFromC::tr("cyl.")).arg(i + 1) +
+				cyl->type.description + ")");
+	}
+	return list;
+}
+
 QString get_taglist_string(struct tag_entry *tag_list)
 {
 	char *buffer = taglist_get_tagstring(tag_list);
@@ -1455,14 +1483,9 @@ extern "C" void parse_display_units(char *line)
 	qDebug() << line;
 }
 
-extern "C" bool in_planner()
+extern "C" enum deco_mode decoMode(bool in_planner)
 {
-	return getAppState() == ApplicationState::PlanDive || getAppState() == ApplicationState::EditPlannedDive;
-}
-
-extern "C" enum deco_mode decoMode()
-{
-	return in_planner() ? prefs.planner_deco_mode : prefs.display_deco_mode;
+	return in_planner ? prefs.planner_deco_mode : prefs.display_deco_mode;
 }
 
 void init_proxy()
@@ -1712,3 +1735,26 @@ extern "C" void emit_reset_signal()
 {
 	emit diveListNotifier.dataReset();
 }
+
+QImage renderSVGIcon(const char *id, int size, bool transparent)
+{
+	QImage res(size, size, transparent ? QImage::Format_ARGB32 : QImage::Format_RGB32);
+	res.fill(transparent ? Qt::transparent : Qt::white);
+	QSvgRenderer svg{QString(id)};
+	QPainter painter(&res);
+	svg.render(&painter);
+	return res;
+}
+
+// As renderSVGIcon(), but render to a fixed width and scale height accordingly
+// and have a transparent background.
+QImage renderSVGIconWidth(const char *id, int size)
+{
+	QSvgRenderer svg{QString(id)};
+	QSize svgSize = svg.defaultSize();
+	QImage res(size, size * svgSize.height() / svgSize.width(), QImage::Format_ARGB32);
+	QPainter painter(&res);
+	svg.render(&painter);
+	return res;
+}
+
