@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "units.h"
 #include "gettext.h"
+#include "pref.h"
 
 int get_pressure_units(int mb, const char **units)
 {
@@ -9,10 +10,6 @@ int get_pressure_units(int mb, const char **units)
 	const struct units *units_p = get_units();
 
 	switch (units_p->pressure) {
-	case PASCALS:
-		pressure = mb * 100;
-		unit = translate("gettextFromC", "pascal");
-		break;
 	case BAR:
 	default:
 		pressure = (mb + 500) / 1000;
@@ -172,3 +169,8 @@ double get_weight_units(unsigned int grams, int *frac, const char **units)
 		*units = unit;
 	return value;
 }
+
+const struct units *get_units()
+{
+	return &prefs.units;
+}
