@@ -50,14 +50,14 @@
 #include "desktop-widgets/tab-widgets/maintab.h"
 #include "desktop-widgets/updatemanager.h"
 #include "desktop-widgets/simplewidgets.h"
+#include "desktop-widgets/statswidget.h"
 #include "commands/command.h"
 
 #include "profile-widget/profilewidget2.h"
 
 #ifndef NO_PRINTING
-#include <QPrintDialog>
-#include <QBuffer>
 #include "desktop-widgets/printdialog.h"
+#include "desktop-widgets/templatelayout.h"
 #endif
 
 #include "qt-models/cylindermodel.h"
@@ -77,10 +77,6 @@ namespace {
 	QProgressDialog *progressDialog = nullptr;
 	bool progressDialogCanceled = false;
 	int progressCounter = 0;
-
-	int round_int (double value) {
-		return static_cast<int>(lrint(value));
-	};
 }
 
 extern "C" int updateProgress(const char *text)
@@ -116,12 +112,12 @@ extern "C" void showErrorFromC(char *buf)
 }
 
 MainWindow::MainWindow() : QMainWindow(),
+	appState((ApplicationState)-1), // Invalid state
 	actionNextDive(nullptr),
 	actionPreviousDive(nullptr),
 #ifndef NO_USERMANUAL
 	helpView(0),
 #endif
-	state(VIEWALL),
 	findMovedImagesDialog(nullptr)
 {
 	Q_ASSERT_X(m_Instance == NULL, "MainWindow", "MainWindow recreated!");
@@ -132,14 +128,17 @@ MainWindow::MainWindow() : QMainWindow(),
 	// Define the States of the Application Here, Currently the states are situations where the different
 	// widgets will change on the mainwindow.
 
+	topSplitter.reset(new QSplitter(Qt::Horizontal));
+	bottomSplitter.reset(new QSplitter(Qt::Horizontal));
+
 	// for the "default" mode
 	mainTab.reset(new MainTab);
-	diveList = new DiveListView(this);
+	diveList.reset(new DiveListView);
 	graphics = new ProfileWidget2(this);
-	MapWidget *mapWidget = MapWidget::instance();
-	divePlannerSettingsWidget = new PlannerSettingsWidget(this);
-	divePlannerWidget = new DivePlannerWidget(this);
-	plannerDetails = new PlannerDetails(this);
+	mapWidget.reset(MapWidget::instance()); // Yes, this is ominous see comment in mapwidget.cpp.
+	plannerWidgets.reset(new PlannerWidgets);
+	statistics.reset(new StatsWidget);
+	profileContainer.reset(new QWidget);
 
 	// what is a sane order for those icons? we should have the ones the user is
 	// most likely to want towards the top so they are always visible
@@ -159,7 +158,6 @@ MainWindow::MainWindow() : QMainWindow(),
 		toolBar->addAction(a);
 	toolBar->setOrientation(Qt::Vertical);
 	toolBar->setIconSize(QSize(24,24));
-	QWidget *profileContainer = new QWidget();
 	QHBoxLayout *profLayout = new QHBoxLayout();
 	profLayout->setSpacing(0);
 	profLayout->setMargin(0);
@@ -168,34 +166,39 @@ MainWindow::MainWindow() : QMainWindow(),
 	profLayout->addWidget(graphics);
 	profileContainer->setLayout(profLayout);
 
-	diveSiteEdit = new LocationInformationWidget(this);
+	diveSiteEdit.reset(new LocationInformationWidget);
 
-	registerApplicationState(ApplicationState::Default, { { mainTab.get(), FLAG_NONE }, { profileContainer, FLAG_NONE },
-							      { diveList, FLAG_NONE },      { mapWidget, FLAG_NONE } });
-	registerApplicationState(ApplicationState::EditDive, { { mainTab.get(), FLAG_NONE }, { profileContainer, FLAG_NONE },
-							       { diveList, FLAG_NONE },       { mapWidget, FLAG_NONE } });
-	registerApplicationState(ApplicationState::PlanDive, { { divePlannerWidget, FLAG_NONE },         { profileContainer, FLAG_NONE },
-							       { divePlannerSettingsWidget, FLAG_NONE }, { plannerDetails, FLAG_NONE } });
-	registerApplicationState(ApplicationState::EditPlannedDive, { { divePlannerWidget, FLAG_NONE }, { profileContainer, FLAG_NONE },
-								      { diveList, FLAG_NONE },          { mapWidget, FLAG_NONE } });
-	registerApplicationState(ApplicationState::EditDiveSite, { { diveSiteEdit, FLAG_NONE }, { profileContainer, FLAG_DISABLED },
-								   { diveList, FLAG_DISABLED }, { mapWidget, FLAG_NONE } });
-	registerApplicationState(ApplicationState::FilterDive, { { mainTab.get(), FLAG_NONE }, { profileContainer, FLAG_NONE },
-								 { diveList, FLAG_NONE },      { &filterWidget, FLAG_NONE } });
+	registerApplicationState(ApplicationState::Default, { true, { mainTab.get(), FLAG_NONE },  { profileContainer.get(), FLAG_NONE },
+								    { diveList.get(), FLAG_NONE }, { mapWidget.get(), FLAG_NONE } });
+	registerApplicationState(ApplicationState::EditDive, { false, { mainTab.get(), FLAG_NONE },  { profileContainer.get(), FLAG_NONE },
+								      { diveList.get(), FLAG_NONE }, { mapWidget.get(), FLAG_NONE } });
+	registerApplicationState(ApplicationState::PlanDive, { false, { &plannerWidgets->plannerWidget, FLAG_NONE },         { profileContainer.get(), FLAG_NONE },
+								      { &plannerWidgets->plannerSettingsWidget, FLAG_NONE }, { &plannerWidgets->plannerDetails, FLAG_NONE } });
+	registerApplicationState(ApplicationState::EditPlannedDive, { true, { &plannerWidgets->plannerWidget, FLAG_NONE }, { profileContainer.get(), FLAG_NONE },
+									    { diveList.get(), FLAG_NONE },                 { mapWidget.get(), FLAG_NONE } });
+	registerApplicationState(ApplicationState::EditDiveSite, { false, { diveSiteEdit.get(), FLAG_NONE }, { profileContainer.get(), FLAG_DISABLED },
+									  { diveList.get(), FLAG_DISABLED }, { mapWidget.get(), FLAG_NONE } });
+	registerApplicationState(ApplicationState::FilterDive, { true, { mainTab.get(), FLAG_NONE },  { profileContainer.get(), FLAG_NONE },
+								       { diveList.get(), FLAG_NONE }, { &filterWidget, FLAG_NONE } });
+	registerApplicationState(ApplicationState::Statistics, { true, { statistics.get(), FLAG_NONE }, { nullptr, FLAG_NONE },
+								       { diveList.get(), FLAG_DISABLED },   { &filterWidget, FLAG_NONE } });
+	registerApplicationState(ApplicationState::MapMaximized, { true, { nullptr, FLAG_NONE }, { nullptr, FLAG_NONE },
+									 { nullptr, FLAG_NONE }, { mapWidget.get(), FLAG_NONE } });
+	registerApplicationState(ApplicationState::ProfileMaximized, { true, { nullptr, FLAG_NONE }, { profileContainer.get(), FLAG_NONE },
+									     { nullptr, FLAG_NONE }, { nullptr, FLAG_NONE } });
+	registerApplicationState(ApplicationState::ListMaximized, { true, { nullptr, FLAG_NONE },        { nullptr, FLAG_NONE },
+									  { diveList.get(), FLAG_NONE }, { nullptr, FLAG_NONE } });
+	registerApplicationState(ApplicationState::InfoMaximized, { true, { mainTab.get(), FLAG_NONE }, { nullptr, FLAG_NONE },
+									  { nullptr, FLAG_NONE },       { nullptr, FLAG_NONE } });
+	restoreSplitterSizes();
 	setApplicationState(ApplicationState::Default);
 
 	setWindowIcon(QIcon(":subsurface-icon"));
 	if (!QIcon::hasThemeIcon("window-close")) {
 		QIcon::setThemeName("subsurface");
 	}
-	connect(diveList, &DiveListView::divesSelected, this, &MainWindow::selectionChanged);
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), this, SLOT(readSettings()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveList, SLOT(update()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), diveList, SLOT(reloadHeaderActions()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), mainTab.get(), SLOT(updateDiveInfo()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerWidget, SLOT(settingsChanged()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), divePlannerSettingsWidget, SLOT(settingsChanged()));
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), TankInfoModel::instance(), SLOT(update()));
+	connect(diveList.get(), &DiveListView::divesSelected, this, &MainWindow::selectionChanged);
+	connect(&diveListNotifier, &DiveListNotifier::settingsChanged, this, &MainWindow::readSettings);
 	for (int i = 0; i < NUM_RECENT_FILES; i++) {
 		actionsRecent[i] = new QAction(this);
 		actionsRecent[i]->setData(i);
@@ -205,7 +208,6 @@ MainWindow::MainWindow() : QMainWindow(),
 	ui.menuFile->insertSeparator(ui.actionQuit);
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCreated()), this, SLOT(planCreated()));
 	connect(DivePlannerPointsModel::instance(), SIGNAL(planCanceled()), this, SLOT(planCanceled()));
-	connect(plannerDetails->printPlan(), SIGNAL(pressed()), divePlannerWidget, SLOT(printDecoPlan()));
 	connect(this, &MainWindow::showError, ui.mainErrorMessage, &NotificationWidget::showError, Qt::AutoConnection);
 
 	connect(&windowTitleUpdate, &WindowTitleUpdate::updateTitle, this, &MainWindow::setAutomaticTitle);
@@ -216,7 +218,6 @@ MainWindow::MainWindow() : QMainWindow(),
 		connect(&diveListNotifier, &DiveListNotifier::divesChanged, this, &MainWindow::divesChanged);
 
 #ifdef NO_PRINTING
-	plannerDetails->printPlan()->hide();
 	ui.menuFile->removeAction(ui.actionPrint);
 #endif
 	enableDisableCloudActions();
@@ -225,14 +226,10 @@ MainWindow::MainWindow() : QMainWindow(),
 	graphics->setEmptyState();
 	initialUiSetup();
 	readSettings();
-	diveList->reload();
-	diveList->reloadHeaderActions();
 	diveList->setFocus();
 	MapWidget::instance()->reload();
 	diveList->expand(diveList->model()->index(0, 0));
 	diveList->scrollTo(diveList->model()->index(0, 0), QAbstractItemView::PositionAtCenter);
-	divePlannerWidget->settingsChanged();
-	divePlannerSettingsWidget->settingsChanged();
 #ifdef NO_USERMANUAL
 	ui.menuHelp->removeAction(ui.actionUserManual);
 #endif
@@ -298,33 +295,13 @@ MainWindow::MainWindow() : QMainWindow(),
 	connect(ui.profPn2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_pn2);
 	connect(ui.profPO2, &QAction::triggered, pp_gas, &qPrefPartialPressureGas::set_po2);
 
-	connect(tec, &qPrefTechnicalDetails::calcalltissuesChanged        , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcceilingChanged           , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::dcceilingChanged             , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::eadChanged                   , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcceiling3mChanged         , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::modChanged                   , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::calcndlttsChanged            , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::hrgraphChanged               , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::rulergraphChanged            , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::show_sacChanged              , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::zoomed_plotChanged           , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::show_pictures_in_profileChanged , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::tankbarChanged               , graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(tec, &qPrefTechnicalDetails::percentagegraphChanged       , graphics, &ProfileWidget2::actionRequestedReplot);
-
-	connect(pp_gas, &qPrefPartialPressureGas::pheChanged, graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(pp_gas, &qPrefPartialPressureGas::pn2Changed, graphics, &ProfileWidget2::actionRequestedReplot);
-	connect(pp_gas, &qPrefPartialPressureGas::po2Changed, graphics, &ProfileWidget2::actionRequestedReplot);
-
 	// now let's set up some connections
 	connect(graphics, &ProfileWidget2::enableToolbar ,this, &MainWindow::setEnabledToolbar);
 	connect(graphics, &ProfileWidget2::disableShortcuts, this, &MainWindow::disableShortcuts);
 	connect(graphics, &ProfileWidget2::enableShortcuts, this, &MainWindow::enableShortcuts);
 	connect(graphics, &ProfileWidget2::editCurrentDive, this, &MainWindow::editCurrentDive);
-	connect(graphics, &ProfileWidget2::updateDiveInfo, mainTab.get(), &MainTab::updateDiveInfo);
 
-	connect(PreferencesDialog::instance(), SIGNAL(settingsChanged()), graphics, SLOT(settingsChanged()));
+	connect(&diveListNotifier, &DiveListNotifier::settingsChanged, graphics, &ProfileWidget2::settingsChanged);
 
 	ui.profCalcAllTissues->setChecked(qPrefTechnicalDetails::calcalltissues());
 	ui.profCalcCeiling->setChecked(qPrefTechnicalDetails::calcceiling());
@@ -354,8 +331,21 @@ MainWindow::MainWindow() : QMainWindow(),
 #endif
 }
 
+static void clearSplitter(QSplitter &splitter)
+{
+	// Qt's ownership model is absolutely hare-brained.
+	// To remove a widget from a splitter, you reparent it, which
+	// informs the splitter via a signal. Wow.
+	while (splitter.count() > 0)
+		splitter.widget(0)->setParent(nullptr);
+}
+
 MainWindow::~MainWindow()
 {
+	// Remove widgets from the splitters so that they don't delete singletons.
+	clearSplitter(*topSplitter);
+	clearSplitter(*bottomSplitter);
+	clearSplitter(*ui.mainSplitter);
 	write_hashes();
 	m_Instance = nullptr;
 }
@@ -394,8 +384,6 @@ void MainWindow::enableDisableOtherDCsActions()
 void MainWindow::setDefaultState()
 {
 	setApplicationState(ApplicationState::Default);
-	if (mainTab->isEditing())
-		ui.bottomLeft->currentWidget()->setEnabled(false);
 }
 
 MainWindow *MainWindow::instance()
@@ -666,7 +654,8 @@ void MainWindow::updateLastUsedDir(const QString &dir)
 void MainWindow::on_actionPrint_triggered()
 {
 #ifndef NO_PRINTING
-	PrintDialog dlg(this);
+	bool in_planner = appState == ApplicationState::PlanDive || appState == ApplicationState::EditPlannedDive;
+	PrintDialog dlg(in_planner, this);
 
 	dlg.exec();
 #endif
@@ -769,113 +758,31 @@ void MainWindow::planCreated()
 	diveList->setFocus();
 }
 
-void MainWindow::setPlanNotes(QString plan)
-{
-	plannerDetails->divePlanOutput()->setHtml(plan);
-}
-
-void MainWindow::printPlan()
-{
-#ifndef NO_PRINTING
-	char *disclaimer = get_planner_disclaimer_formatted();
-	// Prepend a logo and a disclaimer to the plan.
-	// Save the old plan so that it can be restored at the end of the function.
-	QString origPlan = plannerDetails->divePlanOutput()->toHtml();
-	QString diveplan = QStringLiteral("<img height=50 src=\":subsurface-icon\"> ") +
-			   QString(disclaimer) + origPlan;
-	free(disclaimer);
-
-	QPrinter printer;
-	QPrintDialog *dialog = new QPrintDialog(&printer, this);
-	dialog->setWindowTitle(tr("Print runtime table"));
-	if (dialog->exec() != QDialog::Accepted)
-		return;
-
-	/* render the profile as a pixmap that is inserted as base64 data into a HTML <img> tag
-	 * make it fit a page width defined by 2 cm margins via QTextDocument->print() (cannot be changed?)
-	 * the height of the profile is 40% of the page height.
-	 */
-	QSizeF renderSize = printer.pageRect(QPrinter::Inch).size();
-	const qreal marginsInch = 1.57480315; // = (2 x 2cm) / 2.45cm/inch
-	renderSize.setWidth((renderSize.width() - marginsInch) * printer.resolution());
-	renderSize.setHeight(((renderSize.height() - marginsInch) * printer.resolution()) / 2.5);
-
-	QPixmap pixmap(renderSize.toSize());
-	QPainter painter(&pixmap);
-	painter.setRenderHint(QPainter::Antialiasing);
-	painter.setRenderHint(QPainter::SmoothPixmapTransform);
-
-	ProfileWidget2 *profile = graphics;
-	QSize origSize = profile->size();
-	profile->resize(renderSize.toSize());
-	profile->setPrintMode(true);
-	profile->render(&painter);
-	profile->resize(origSize);
-	profile->setPrintMode(false);
-
-	QByteArray byteArray;
-	QBuffer buffer(&byteArray);
-	pixmap.save(&buffer, "PNG");
-	QString profileImage = QString("<img src=\"data:image/png;base64,") + byteArray.toBase64() + "\"/><br><br>";
-	diveplan = profileImage + diveplan;
-
-	plannerDetails->divePlanOutput()->setHtml(diveplan);
-	plannerDetails->divePlanOutput()->print(&printer);
-	plannerDetails->divePlanOutput()->setHtml(origPlan); // restore original plan
-#endif
-}
-
 void MainWindow::on_actionReplanDive_triggered()
 {
-	if (!plannerStateClean() || !current_dive)
+	if (!plannerStateClean() || !current_dive || !userMayChangeAppState())
 		return;
 	else if (!is_dc_planner(&current_dive->dc)) {
 		if (QMessageBox::warning(this, tr("Warning"), tr("Trying to replan a dive that's not a planned dive."),
 					 QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Cancel)
 					return;
 	}
-	// put us in PLAN mode
-	DivePlannerPointsModel::instance()->clear();
-	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 
-	graphics->setPlanState();
-	graphics->clearHandlers();
+	// put us in PLAN mode
 	setApplicationState(ApplicationState::PlanDive);
-	divePlannerWidget->setReplanButton(true);
-	divePlannerWidget->setupStartTime(timestampToDateTime(current_dive->when));
-	if (current_dive->surface_pressure.mbar)
-		divePlannerWidget->setSurfacePressure(current_dive->surface_pressure.mbar);
-	if (current_dive->salinity)
-		divePlannerWidget->setSalinity(current_dive->salinity);
-	DivePlannerPointsModel::instance()->loadFromDive(current_dive);
-	reset_cylinders(&displayed_dive, true);
-	DivePlannerPointsModel::instance()->cylindersModel()->updateDive(&displayed_dive);
+
+	plannerWidgets->replanDive();
 }
 
 void MainWindow::on_actionDivePlanner_triggered()
 {
-	if (!plannerStateClean())
+	if (!plannerStateClean() || !userMayChangeAppState())
 		return;
 
 	// put us in PLAN mode
-	DivePlannerPointsModel::instance()->setPlanMode(DivePlannerPointsModel::PLAN);
 	setApplicationState(ApplicationState::PlanDive);
 
-	graphics->setPlanState();
-	dc_number = 0;
-
-	// create a simple starting dive, using the first gas from the just copied cylinders
-	DivePlannerPointsModel::instance()->createSimpleDive();
-	// plan the dive in the same mode as the currently selected one
-	if (current_dive) {
-		divePlannerSettingsWidget->setDiveMode(current_dive->dc.divemode);
-		divePlannerSettingsWidget->setBailoutVisibility(current_dive->dc.divemode);
-		if (current_dive->salinity)
-			divePlannerWidget->setSalinity(current_dive->salinity);
-		else	// No salinity means salt water
-			divePlannerWidget->setSalinity(SEAWATER_SALINITY);
-	}
-	divePlannerWidget->setReplanButton(false);
+	plannerWidgets->planDive();
 }
 
 void MainWindow::on_actionAddDive_triggered()
@@ -935,125 +842,78 @@ void MainWindow::on_actionYearlyStatistics_triggered()
 	d.exec();
 }
 
-void MainWindow::toggleCollapsible(bool toggle)
-{
-	ui.mainSplitter->setCollapsible(0, toggle);
-	ui.mainSplitter->setCollapsible(1, toggle);
-	ui.topSplitter->setCollapsible(0, toggle);
-	ui.topSplitter->setCollapsible(1, toggle);
-	ui.bottomSplitter->setCollapsible(0, toggle);
-	ui.bottomSplitter->setCollapsible(1, toggle);
-}
-
 void MainWindow::on_actionViewList_triggered()
 {
-	toggleCollapsible(true);
-	beginChangeState(LIST_MAXIMIZED);
-	ui.mainSplitter->setSizes({ COLLAPSED, EXPANDED });
-	showFilterIfEnabled();
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(ApplicationState::ListMaximized);
 }
 
 void MainWindow::on_actionViewProfile_triggered()
 {
-	toggleCollapsible(true);
-	beginChangeState(PROFILE_MAXIMIZED);
-	ui.topSplitter->setSizes({ COLLAPSED, EXPANDED });
-	ui.mainSplitter->setSizes({ EXPANDED, COLLAPSED });
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(ApplicationState::ProfileMaximized);
 }
 
 void MainWindow::on_actionViewInfo_triggered()
 {
-	toggleCollapsible(true);
-	beginChangeState(INFO_MAXIMIZED);
-	ui.topSplitter->setSizes({ EXPANDED, COLLAPSED });
-	ui.mainSplitter->setSizes({ EXPANDED, COLLAPSED });
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(ApplicationState::InfoMaximized);
 }
 
 void MainWindow::on_actionViewMap_triggered()
 {
-	toggleCollapsible(true);
-	beginChangeState(MAP_MAXIMIZED);
-	ui.mainSplitter->setSizes({ COLLAPSED, EXPANDED });
-	ui.bottomSplitter->setSizes({ COLLAPSED, EXPANDED });
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(ApplicationState::MapMaximized);
 }
 
 void MainWindow::on_actionViewAll_triggered()
 {
-	toggleCollapsible(false);
-	beginChangeState(VIEWALL);
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(ApplicationState::Default);
+}
 
-	const int appH = qApp->desktop()->size().height();
-	const int appW = qApp->desktop()->size().width();
+void MainWindow::saveSplitterSizes()
+{
+	// Only save splitters if all four quadrants are shown
+	if (ui.mainSplitter->count() < 2 || topSplitter->count() < 2 || bottomSplitter->count() < 2)
+		return;
+
+	QSettings settings;
+	settings.beginGroup("MainWindow");
+	settings.setValue("mainSplitter", ui.mainSplitter->saveState());
+	settings.setValue("topSplitter", topSplitter->saveState());
+	settings.setValue("bottomSplitter", bottomSplitter->saveState());
+}
+
+void MainWindow::restoreSplitterSizes()
+{
+	// Only restore splitters if all four quadrants are shown
+	if (ui.mainSplitter->count() < 2 || topSplitter->count() < 2 || bottomSplitter->count() < 2)
+		return;
 
-	QList<int> mainSizes = { round_int(appH * 0.7), round_int(appH * 0.3) };
-	QList<int> infoProfileSizes = { round_int(appW * 0.3), round_int(appW * 0.7) };
-	QList<int> listGlobeSizes = { round_int(appW * 0.7), round_int(appW * 0.3) };
 
 	QSettings settings;
 	settings.beginGroup("MainWindow");
 	if (settings.value("mainSplitter").isValid()) {
 		ui.mainSplitter->restoreState(settings.value("mainSplitter").toByteArray());
-		ui.topSplitter->restoreState(settings.value("topSplitter").toByteArray());
-		ui.bottomSplitter->restoreState(settings.value("bottomSplitter").toByteArray());
-		if (ui.mainSplitter->sizes().first() == 0 || ui.mainSplitter->sizes().last() == 0)
-			ui.mainSplitter->setSizes(mainSizes);
-		if (ui.topSplitter->sizes().first() == 0 || ui.topSplitter->sizes().last() == 0)
-			ui.topSplitter->setSizes(infoProfileSizes);
-		if (ui.bottomSplitter->sizes().first() == 0 || ui.bottomSplitter->sizes().last() == 0)
-			ui.bottomSplitter->setSizes(listGlobeSizes);
 
+		topSplitter->restoreState(settings.value("topSplitter").toByteArray());
+		bottomSplitter->restoreState(settings.value("bottomSplitter").toByteArray());
 	} else {
-		ui.mainSplitter->setSizes(mainSizes);
-		ui.topSplitter->setSizes(infoProfileSizes);
-		ui.bottomSplitter->setSizes(listGlobeSizes);
-	}
-	ui.mainSplitter->setCollapsible(0, false);
-	ui.mainSplitter->setCollapsible(1, false);
-	ui.topSplitter->setCollapsible(0, false);
-	ui.topSplitter->setCollapsible(1, false);
-	ui.bottomSplitter->setCollapsible(0,false);
-	ui.bottomSplitter->setCollapsible(1,false);
-}
+		const int appH = qApp->desktop()->size().height();
+		const int appW = qApp->desktop()->size().width();
 
-void MainWindow::enterState(CurrentState newState)
-{
-	state = newState;
-	switch (state) {
-	case VIEWALL:
-		on_actionViewAll_triggered();
-		break;
-	case MAP_MAXIMIZED:
-		on_actionViewMap_triggered();
-		break;
-	case INFO_MAXIMIZED:
-		on_actionViewInfo_triggered();
-		break;
-	case LIST_MAXIMIZED:
-		on_actionViewList_triggered();
-		break;
-	case PROFILE_MAXIMIZED:
-		on_actionViewProfile_triggered();
-		break;
+		ui.mainSplitter->setSizes({ appH * 3 / 5, appH * 2 / 5 });
+		topSplitter->setSizes({ appW / 2, appW / 2 });
+		bottomSplitter->setSizes({ appW * 3 / 5, appW * 2 / 5 });
 	}
 }
 
-void MainWindow::beginChangeState(CurrentState s)
-{
-	if (state == VIEWALL && state != s) {
-		saveSplitterSizes();
-	}
-	state = s;
-}
-
-void MainWindow::saveSplitterSizes()
-{
-	QSettings settings;
-	settings.beginGroup("MainWindow");
-	settings.setValue("mainSplitter", ui.mainSplitter->saveState());
-	settings.setValue("topSplitter", ui.topSplitter->saveState());
-	settings.setValue("bottomSplitter", ui.bottomSplitter->saveState());
-}
-
 void MainWindow::on_actionPreviousDC_triggered()
 {
 	unsigned nrdc = number_of_computers(current_dive);
@@ -1247,7 +1107,7 @@ void MainWindow::initialUiSetup()
 		restoreState(settings.value("windowState", 0).toByteArray());
 	}
 
-	enterState((CurrentState)settings.value("lastState", 0).toInt());
+	setApplicationState((ApplicationState)settings.value("lastAppState", 0).toInt());
 	settings.endGroup();
 	show();
 }
@@ -1272,9 +1132,8 @@ void MainWindow::writeSettings()
 	settings.setValue("geometry", saveGeometry());
 	settings.setValue("windowState", saveState());
 	settings.setValue("maximized", isMaximized());
-	settings.setValue("lastState", (int)state);
-	if (state == VIEWALL)
-		saveSplitterSizes();
+	settings.setValue("lastState", (int)appState);
+	saveSplitterSizes();
 	settings.endGroup();
 }
 
@@ -1647,7 +1506,7 @@ void MainWindow::on_actionImportDiveSites_triggered()
 void MainWindow::editCurrentDive()
 {
 	// We only allow editing of the profile for manually added dives.
-	if (!current_dive || !same_string(current_dive->dc.model, "manually added dive"))
+	if (!current_dive || !same_string(current_dive->dc.model, "manually added dive") || !userMayChangeAppState())
 		return;
 
 	// This shouldn't be possible, but let's make sure no weird "double editing" takes place.
@@ -1668,9 +1527,6 @@ void MainWindow::turnOffNdlTts()
 	qPrefTechnicalDetails::set_calcndltts(false);
 }
 
-#undef TOOLBOX_PREF_PROFILE
-#undef PERF_PROFILE
-
 void MainWindow::on_actionExport_triggered()
 {
 	DiveLogExportDialog diveLogExport;
@@ -1704,60 +1560,99 @@ void MainWindow::on_paste_triggered()
 
 void MainWindow::on_actionFilterTags_triggered()
 {
-	setApplicationState(getAppState() == ApplicationState::FilterDive ? ApplicationState::Default : ApplicationState::FilterDive);
-	if (state == LIST_MAXIMIZED)
-		showFilterIfEnabled();
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(appState == ApplicationState::FilterDive ? ApplicationState::Default : ApplicationState::FilterDive);
 }
 
-void MainWindow::showFilterIfEnabled()
+void MainWindow::on_actionStats_triggered()
 {
-	if (getAppState() == ApplicationState::FilterDive) {
-		const int appW = qApp->desktop()->size().width();
-		QList<int> profileFilterSizes = { round_int(appW * 0.7), round_int(appW * 0.3) };
-		ui.bottomSplitter->setSizes(profileFilterSizes);
-	} else {
-		ui.bottomSplitter->setSizes({ EXPANDED, COLLAPSED });
-	}
-}
-
-void MainWindow::addWidgets(const Quadrant &q, QStackedWidget *stack)
-{
-	if (q.widget && stack->indexOf(q.widget) == -1)
-		stack->addWidget(q.widget);
+	if (!userMayChangeAppState())
+		return;
+	setApplicationState(appState == ApplicationState::Statistics ? ApplicationState::Default : ApplicationState::Statistics);
 }
 
 void MainWindow::registerApplicationState(ApplicationState state, Quadrants q)
 {
 	applicationState[(int)state] = q;
-	addWidgets(q.topLeft, ui.topLeft);
-	addWidgets(q.topRight, ui.topRight);
-	addWidgets(q.bottomLeft, ui.bottomLeft);
-	addWidgets(q.bottomRight, ui.bottomRight);
 }
 
-void MainWindow::setQuadrant(const Quadrant &q, QStackedWidget *stack)
+void MainWindow::setQuadrantWidget(QSplitter &splitter, const Quadrant &q, int pos)
 {
-	if (q.widget) {
-		stack->setCurrentWidget(q.widget);
-		stack->show();
-		q.widget->setEnabled(!(q.flags & FLAG_DISABLED));
-	} else {
-		stack->hide();
-	}
+	if (!q.widget)
+		return;
+	if (splitter.count() > pos)
+		splitter.replaceWidget(pos, q.widget);
+	else
+		splitter.addWidget(q.widget);
+	splitter.setCollapsible(pos, false);
+	q.widget->setEnabled(!(q.flags & FLAG_DISABLED));
+}
+
+void MainWindow::setQuadrantWidgets(QSplitter &splitter, const Quadrant &left, const Quadrant &right)
+{
+	int num = (left.widget != nullptr) + (right.widget != nullptr);
+	// Remove superfluous widgets by reparenting to null.
+	while (splitter.count() > num)
+		splitter.widget(splitter.count() - 1)->setParent(nullptr);
+
+	setQuadrantWidget(splitter, left, 0);
+	setQuadrantWidget(splitter, right, left.widget != nullptr ? 1 : 0);
+}
+
+bool MainWindow::userMayChangeAppState() const
+{
+	return applicationState[(int)appState].allowUserChange;
 }
 
 void MainWindow::setApplicationState(ApplicationState state)
 {
-	if (getAppState() == state)
+	if (appState == state)
 		return;
 
-	setAppState(state);
+	saveSplitterSizes();
 
+	appState = state;
+
+	clearSplitter(*topSplitter);
+	clearSplitter(*bottomSplitter);
 	const Quadrants &quadrants = applicationState[(int)state];
-	setQuadrant(quadrants.topLeft, ui.topLeft);
-	setQuadrant(quadrants.topRight, ui.topRight);
-	setQuadrant(quadrants.bottomLeft, ui.bottomLeft);
-	setQuadrant(quadrants.bottomRight, ui.bottomRight);
+	setQuadrantWidgets(*topSplitter, quadrants.topLeft, quadrants.topRight);
+	setQuadrantWidgets(*bottomSplitter, quadrants.bottomLeft, quadrants.bottomRight);
+
+	if (topSplitter->count() >= 1) {
+		// Add topSplitter if it is not already shown
+		if (ui.mainSplitter->count() == 0 ||
+		    ui.mainSplitter->widget(0) != topSplitter.get()) {
+			ui.mainSplitter->insertWidget(0, topSplitter.get());
+			ui.mainSplitter->setCollapsible(ui.mainSplitter->count() - 1, false);
+		}
+	} else {
+		// Remove topSplitter by reparenting it. So weird.
+		topSplitter->setParent(nullptr);
+	}
+
+	if (bottomSplitter->count() >= 1) {
+		// Add bottomSplitter if it is not already shown
+		if (ui.mainSplitter->count() == 0 ||
+		    ui.mainSplitter->widget(ui.mainSplitter->count() - 1) != bottomSplitter.get()) {
+			ui.mainSplitter->addWidget(bottomSplitter.get());
+			ui.mainSplitter->setCollapsible(ui.mainSplitter->count() - 1, false);
+		}
+	} else {
+		// Remove bottomSplitter by reparenting it. So weird.
+		bottomSplitter->setParent(nullptr);
+	}
+
+	restoreSplitterSizes();
+
+	bool allowChange = userMayChangeAppState();
+	ui.actionViewAll->setEnabled(allowChange);
+	ui.actionViewList->setEnabled(allowChange);
+	ui.actionViewProfile->setEnabled(allowChange);
+	ui.actionViewInfo->setEnabled(allowChange);
+	ui.actionViewMap->setEnabled(allowChange);
+	ui.actionFilterTags->setEnabled(allowChange);
 }
 
 void MainWindow::showProgressBar()
