@@ -38,6 +38,7 @@
 #include "core/downloadfromdcthread.h"
 #include "core/subsurfacestartup.h" // for ignore_bt flag
 #include "core/subsurface-string.h"
+#include "core/string-format.h"
 #include "core/pref.h"
 #include "core/selection.h"
 #include "core/ssrf.h"
@@ -47,7 +48,6 @@
 #include "core/settings/qPrefTechnicalDetails.h"
 #include "core/settings/qPrefPartialPressureGas.h"
 #include "core/settings/qPrefUnit.h"
-#include "core/subsurface-qt/diveobjecthelper.h"
 #include "core/trip.h"
 #include "backend-shared/exportfuncs.h"
 #include "core/worldmap-save.h"
@@ -128,7 +128,12 @@ static void showProgress(QString msg)
 // show the git progress in the passive notification area
 extern "C" int gitProgressCB(const char *text)
 {
-	showProgress(QString(text));
+	// regular users, during regular operation, likely really don't
+	// care at all about the git progress
+	if (verbose) {
+		showProgress(QString(text));
+		appendTextToLogStandalone(text);
+	}
 	// return 0 so that we don't end the download
 	return 0;
 }
@@ -209,7 +214,6 @@ QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	m_instance = this;
 	m_lastDevicePixelRatio = qApp->devicePixelRatio();
 	timer.start();
-	connect(qobject_cast<QApplication *>(QApplication::instance()), &QApplication::applicationStateChanged, this, &QMLManager::applicationStateChanged);
 
 	// make upload signals available in QML
 	// Remark: signal - signal connect
@@ -290,11 +294,11 @@ QMLManager::QMLManager() : m_locationServiceEnabled(false),
 		connect(&btDiscovery->localBtDevice, &QBluetoothLocalDevice::hostModeStateChanged,
 			this, &QMLManager::btHostModeChange);
 	}
-	// create location manager service
-	locationProvider = new GpsLocation(&appendTextToLogStandalone, this);
+	// add log call back to the location manager service singleton
+	GpsLocation::instance()->setLogCallBack(&appendTextToLogStandalone);
 	progress_callback = &progressCallback;
-	connect(locationProvider, SIGNAL(haveSourceChanged()), this, SLOT(hasLocationSourceChanged()));
-	setLocationServiceAvailable(locationProvider->hasLocationsSource());
+	connect(GpsLocation::instance(), &GpsLocation::haveSourceChanged, this, &QMLManager::hasLocationSourceChanged);
+	setLocationServiceAvailable(GpsLocation::instance()->hasLocationsSource());
 	set_git_update_cb(&gitProgressCB);
 
 	// present dive site lists sorted by name
@@ -324,8 +328,14 @@ QMLManager::QMLManager() : m_locationServiceEnabled(false),
 	connect(Command::getUndoStack(), &QUndoStack::undoTextChanged, this, &QMLManager::undoTextChanged);
 	connect(Command::getUndoStack(), &QUndoStack::redoTextChanged, this, &QMLManager::redoTextChanged);
 
+	// now that everything is setup, connect the application changed signal
+	connect(qobject_cast<QApplication *>(QApplication::instance()), &QApplication::applicationStateChanged, this, &QMLManager::applicationStateChanged);
+
 	// we start out with clean data
 	updateHaveLocalChanges(false);
+
+	// setup Command infrastructure
+	Command::init();
 }
 
 void QMLManager::applicationStateChanged(Qt::ApplicationState state)
@@ -482,6 +492,16 @@ void QMLManager::copyAppLogToClipboard()
 	QApplication::clipboard()->setText(getCombinedLogs(), QClipboard::Clipboard);
 }
 
+void QMLManager::copyGpsFixesToClipboard()
+{
+	// This of course creates a potential privacy issue, so let's be clear about that
+	QString gpsWarning("Sending these GPS data to someone exposes your location history; ");
+	gpsWarning += "they can, however, be helpful when debugging problems with the app. ";
+	gpsWarning += "Please consider carefully where you are seninding these data.\n\n";
+	gpsWarning += GpsLocation::instance()->getFixString();
+	QApplication::clipboard()->setText(gpsWarning, QClipboard::Clipboard);
+}
+
 bool QMLManager::createSupportEmail()
 {
 	QString mailToLink = "mailto:in-app-support@subsurface-divelog.org?subject=Subsurface-mobile support request";
@@ -851,9 +871,9 @@ static void setupDivesite(DiveSiteChange &res, struct dive *d, struct dive_site
 	res.changed = true;
 }
 
-bool QMLManager::checkDate(const DiveObjectHelper &myDive, struct dive *d, QString date)
+bool QMLManager::checkDate(struct dive *d, QString date)
 {
-	QString oldDate = myDive.date() + " " + myDive.time();
+	QString oldDate = formatDiveDateTime(d);
 	if (date != oldDate) {
 		QDateTime newDate;
 		// what a pain - Qt will not parse dates if the day of the week is incorrect
@@ -956,12 +976,12 @@ parsed:
 	return false;
 }
 
-bool QMLManager::checkLocation(DiveSiteChange &res, const DiveObjectHelper &myDive, struct dive *d, QString location, QString gps)
+bool QMLManager::checkLocation(DiveSiteChange &res, struct dive *d, QString location, QString gps)
 {
 	struct dive_site *ds = get_dive_site_for_dive(d);
 	bool changed = false;
-	qDebug() << "checkLocation" << location << "gps" << gps << "dive had" << myDive.location << "gps" << myDive.gas;
-	if (myDive.location != location) {
+	QString oldLocation = get_dive_location(d);
+	if (oldLocation != location) {
 		ds = get_dive_site_by_name(qPrintable(location), &dive_site_table);
 		if (!ds && !location.isEmpty()) {
 			res.createdDs.reset(alloc_dive_site_with_name(qPrintable(location)));
@@ -974,7 +994,7 @@ bool QMLManager::checkLocation(DiveSiteChange &res, const DiveObjectHelper &myDi
 	// now make sure that the GPS coordinates match - if the user changed the name but not
 	// the GPS coordinates, this still does the right thing as the now new dive site will
 	// have no coordinates, so the coordinates from the edit screen will get added
-	if (myDive.gps != gps) {
+	if (formatDiveGPS(d) != gps) {
 		double lat, lon;
 		if (parseGpsText(gps, &lat, &lon)) {
 			qDebug() << "parsed GPS, using it";
@@ -999,9 +1019,9 @@ bool QMLManager::checkLocation(DiveSiteChange &res, const DiveObjectHelper &myDi
 	return changed | res.changed;
 }
 
-bool QMLManager::checkDuration(const DiveObjectHelper &myDive, struct dive *d, QString duration)
+bool QMLManager::checkDuration(struct dive *d, QString duration)
 {
-	if (myDive.duration != duration) {
+	if (formatDiveDuration(d) != duration) {
 		int h = 0, m = 0, s = 0;
 		QRegExp r1(QStringLiteral("(\\d*)\\s*%1[\\s,:]*(\\d*)\\s*%2[\\s,:]*(\\d*)\\s*%3").arg(tr("h")).arg(tr("min")).arg(tr("sec")), Qt::CaseInsensitive);
 		QRegExp r2(QStringLiteral("(\\d*)\\s*%1[\\s,:]*(\\d*)\\s*%2").arg(tr("h")).arg(tr("min")), Qt::CaseInsensitive);
@@ -1038,9 +1058,9 @@ bool QMLManager::checkDuration(const DiveObjectHelper &myDive, struct dive *d, Q
 	return false;
 }
 
-bool QMLManager::checkDepth(const DiveObjectHelper &myDive, dive *d, QString depth)
+bool QMLManager::checkDepth(dive *d, QString depth)
 {
-	if (myDive.depth != depth) {
+	if (get_depth_string(d->dc.maxdepth.mm, true, true) != depth) {
 		int depthValue = parseLengthToMm(depth);
 		// the QML code should stop negative depth, but massively huge depth can make
 		// the profile extremely slow or even run out of memory and crash, so keep
@@ -1068,11 +1088,26 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		appendTextToLog("cannot commit changes: no dive");
 		return;
 	}
+	if (verbose) {
+		qDebug().noquote() << QStringLiteral("diveId  :'%1'\n").arg(diveId) <<
+				      QStringLiteral("number  :'%1'\n").arg(number) <<
+				      QStringLiteral("date    :'%1'\n").arg(date) <<
+				      QStringLiteral("location:'%1'\n").arg(location) <<
+				      QStringLiteral("gps     :'%1'\n").arg(gps) <<
+				      QStringLiteral("duration:'%1'\n").arg(duration) <<
+				      QStringLiteral("depth   :'%1'\n").arg(depth) <<
+				      QStringLiteral("airtemp :'%1'\n").arg(airtemp) <<
+				      QStringLiteral("watertmp:'%1'\n").arg(watertemp) <<
+				      QStringLiteral("suit    :'%1'\n").arg(suit) <<
+				      QStringLiteral("buddy   :'%1'\n").arg(buddy) <<
+				      QStringLiteral("diveMstr:'%1'\n").arg(diveMaster) <<
+				      QStringLiteral("weight  :'%1'\n").arg(weight) <<
+				      QStringLiteral("state   :'%1'\n").arg(state);
+	}
 
 	Command::OwningDivePtr d_ptr(alloc_dive()); // Automatically delete dive if we exit early!
 	dive *d = d_ptr.get();
 	copy_dive(orig, d);
-	DiveObjectHelper myDive(d);
 
 	// notes comes back as rich text - let's convert this into plain text
 	QTextDocument doc;
@@ -1081,28 +1116,28 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 
 	bool diveChanged = false;
 
-	diveChanged = checkDate(myDive, d, date);
+	diveChanged = checkDate(d, date);
 
 	DiveSiteChange dsChange;
-	diveChanged |= checkLocation(dsChange, myDive, d, location, gps);
+	diveChanged |= checkLocation(dsChange, d, location, gps);
 
-	diveChanged |= checkDuration(myDive, d, duration);
+	diveChanged |= checkDuration(d, duration);
 
-	diveChanged |= checkDepth(myDive, d, depth);
+	diveChanged |= checkDepth(d, depth);
 
-	if (QString::number(myDive.number) != number) {
+	if (QString::number(d->number) != number) {
 		diveChanged = true;
 		d->number = number.toInt();
 	}
-	if (myDive.airTemp != airtemp) {
+	if (get_temperature_string(d->airtemp, true) != airtemp) {
 		diveChanged = true;
 		d->airtemp.mkelvin = parseTemperatureToMkelvin(airtemp);
 	}
-	if (myDive.waterTemp != watertemp) {
+	if (get_temperature_string(d->watertemp, true) != watertemp) {
 		diveChanged = true;
 		d->watertemp.mkelvin = parseTemperatureToMkelvin(watertemp);
 	}
-	if (myDive.sumWeight != weight) {
+	if (formatSumWeight(d) != weight) {
 		diveChanged = true;
 		// not sure what we'd do if there was more than one weight system
 		// defined - for now just ignore that case
@@ -1119,7 +1154,7 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		startpressure = QStringList();
 	if (endpressure == QStringList(QString()))
 		endpressure = QStringList();
-	if (myDive.startPressure != startpressure || myDive.endPressure != endpressure) {
+	if (formatStartPressure(d) != startpressure || formatEndPressure(d) != endpressure) {
 		diveChanged = true;
 		for ( int i = 0, j = 0 ; j < startpressure.length() && j < endpressure.length() ; i++ ) {
 			if (state != "add" && !is_cylinder_used(d, i))
@@ -1135,7 +1170,7 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		}
 	}
 	// gasmix for first cylinder
-	if (myDive.firstGas != gasmix) {
+	if (formatFirstGas(d) != gasmix) {
 		for ( int i = 0, j = 0 ; j < gasmix.length() ; i++ ) {
 			if (state != "add" && !is_cylinder_used(d, i))
 				continue;
@@ -1154,22 +1189,22 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		}
 	}
 	// info for first cylinder
-	if (myDive.getCylinder != usedCylinder) {
+	if (formatGetCylinder(d) != usedCylinder) {
 		diveChanged = true;
-		unsigned long i;
 		int size = 0, wp = 0, j = 0, k = 0;
 		for (j = 0; k < usedCylinder.length(); j++) {
 			if (state != "add" && !is_cylinder_used(d, j))
 				continue;
 
-			for (i = 0; i < MAX_TANK_INFO && tank_info[i].name != NULL; i++) {
-				if (tank_info[i].name == usedCylinder[k] ) {
-					if (tank_info[i].ml > 0){
-						size = tank_info[i].ml;
-						wp = tank_info[i].bar * 1000;
+			for (int i = 0; i < tank_info_table.nr; i++) {
+				const tank_info &ti = tank_info_table.infos[i];
+				if (ti.name == usedCylinder[k] ) {
+					if (ti.ml > 0){
+						size = ti.ml;
+						wp = ti.bar * 1000;
 					} else {
-						size = (int) (cuft_to_l(tank_info[i].cuft) * 1000 / bar_to_atm(psi_to_bar(tank_info[i].psi)));
-						wp = psi_to_mbar(tank_info[i].psi);
+						size = (int) (cuft_to_l(ti.cuft) * 1000 / bar_to_atm(psi_to_bar(ti.psi)));
+						wp = psi_to_mbar(ti.psi);
 					}
 					break;
 				}
@@ -1180,12 +1215,12 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 			k++;
 		}
 	}
-	if (myDive.suit != suit) {
+	if (d->suit != suit) {
 		diveChanged = true;
 		free(d->suit);
 		d->suit = copy_qstring(suit);
 	}
-	if (myDive.buddy != buddy) {
+	if (d->buddy != buddy) {
 		if (buddy.contains(",")){
 			buddy = buddy.replace(QRegExp("\\s*,\\s*"), ", ");
 		}
@@ -1193,7 +1228,7 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		free(d->buddy);
 		d->buddy = copy_qstring(buddy);
 	}
-	if (myDive.divemaster != diveMaster) {
+	if (d->divemaster != diveMaster) {
 		if (diveMaster.contains(",")){
 			diveMaster = diveMaster.replace(QRegExp("\\s*,\\s*"), ", ");
 		}
@@ -1201,15 +1236,15 @@ void QMLManager::commitChanges(QString diveId, QString number, QString date, QSt
 		free(d->divemaster);
 		d->divemaster = copy_qstring(diveMaster);
 	}
-	if (myDive.rating != rating) {
+	if (d->rating != rating) {
 		diveChanged = true;
 		d->rating = rating;
 	}
-	if (myDive.visibility != visibility) {
+	if (d->visibility != visibility) {
 		diveChanged = true;
 		d->visibility = visibility;
 	}
-	if (myDive.notes != notes) {
+	if (formatNotes(d) != notes) {
 		diveChanged = true;
 		free(d->notes);
 		d->notes = copy_qstring(notes);
@@ -1307,7 +1342,7 @@ void QMLManager::addDiveToTrip(int id, int tripId)
 	changesNeedSaving();
 }
 
-void QMLManager::changesNeedSaving()
+void QMLManager::changesNeedSaving(bool fromUndo)
 {
 	// we no longer save right away on iOS because file access is so slow; on the other hand,
 	// on Android the save as the user switches away doesn't seem to work... drat.
@@ -1318,9 +1353,9 @@ void QMLManager::changesNeedSaving()
 	mark_divelist_changed(true);
 	emit syncStateChanged();
 #if defined(Q_OS_IOS)
-	saveChangesLocal();
+	saveChangesLocal(fromUndo);
 #else
-	saveChangesCloud(false);
+	saveChangesCloud(false, fromUndo);
 #endif
 	updateAllGlobalLists();
 }
@@ -1351,7 +1386,7 @@ void QMLManager::openNoCloudRepo()
 	openLocalThenRemote(filename);
 }
 
-void QMLManager::saveChangesLocal()
+void QMLManager::saveChangesLocal(bool fromUndo)
 {
 	if (unsavedChanges()) {
 		if (qPrefCloudStorage::cloud_verification_status() == qPrefCloudStorage::CS_NOCLOUD) {
@@ -1381,12 +1416,21 @@ void QMLManager::saveChangesLocal()
 		mark_divelist_changed(false);
 		Command::setClean();
 		updateHaveLocalChanges(true);
+		// provide a useful undo/redo notification
+		// NOTE: the QML UI interprets a leading '[action]' (where only the two brackets are checked for)
+		//       as an indication to use the text between those two brackets as the label of a button that
+		//       can be used to open the context menu
+		QString msgFormat = tr("[%1]Changes saved:'%2'.\n%1 possible via context menu");
+		if (fromUndo)
+			setNotificationText(msgFormat.arg(tr("Redo")).arg(tr("Undo: %1").arg(getRedoText())));
+		else
+			setNotificationText(msgFormat.arg(tr("Undo")).arg(getUndoText()));
 	} else {
 		appendTextToLog("local save requested with no unsaved changes");
 	}
 }
 
-void QMLManager::saveChangesCloud(bool forceRemoteSync)
+void QMLManager::saveChangesCloud(bool forceRemoteSync, bool fromUndo)
 {
 	if (!unsavedChanges() && !forceRemoteSync) {
 		appendTextToLog("asked to save changes but no unsaved changes");
@@ -1394,7 +1438,7 @@ void QMLManager::saveChangesCloud(bool forceRemoteSync)
 	}
 	// first we need to store any unsaved changes to the local repo
 	gitProgressCB("Save changes to local cache");
-	saveChangesLocal();
+	saveChangesLocal(fromUndo);
 	// if the user asked not to push to the cloud we are done
 	if (git_local_only && !forceRemoteSync)
 		return;
@@ -1414,7 +1458,7 @@ void QMLManager::saveChangesCloud(bool forceRemoteSync)
 void QMLManager::undo()
 {
 	Command::getUndoStack()->undo();
-	changesNeedSaving();
+	changesNeedSaving(true);
 }
 
 void QMLManager::redo()
@@ -1596,25 +1640,25 @@ int QMLManager::addDive()
 QString QMLManager::getCurrentPosition()
 {
 	static bool hasLocationSource = false;
-	if (locationProvider->hasLocationsSource() != hasLocationSource) {
+	if (GpsLocation::instance()->hasLocationsSource() != hasLocationSource) {
 		hasLocationSource = !hasLocationSource;
 		setLocationServiceAvailable(hasLocationSource);
 	}
 	if (!hasLocationSource)
 		return tr("Unknown GPS location");
 
-	QString positionResponse = locationProvider->currentPosition();
+	QString positionResponse = GpsLocation::instance()->currentPosition();
 	if (positionResponse == GPS_CURRENT_POS)
-		connect(locationProvider, &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged, Qt::UniqueConnection);
+		connect(GpsLocation::instance(), &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged, Qt::UniqueConnection);
 	else
-		disconnect(locationProvider, &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged);
+		disconnect(GpsLocation::instance(), &GpsLocation::acquiredPosition, this, &QMLManager::waitingForPositionChanged);
 	return positionResponse;
 }
 
 void QMLManager::applyGpsData()
 {
 	appendTextToLog("Applying GPS fiexs");
-	std::vector<DiveAndLocation> fixes = locationProvider->getLocations();
+	std::vector<DiveAndLocation> fixes = GpsLocation::instance()->getLocations();
 	Command::applyGPSFixes(fixes);
 	appendTextToLog(QString("Attached %1 GPS fixes").arg(fixes.size()));
 	if (fixes.size())
@@ -1629,19 +1673,19 @@ void QMLManager::populateGpsData()
 
 void QMLManager::clearGpsData()
 {
-	locationProvider->clearGpsData();
+	GpsLocation::instance()->clearGpsData();
 	populateGpsData();
 }
 
 void QMLManager::deleteGpsFix(quint64 when)
 {
-	locationProvider->deleteGpsFix(when);
+	GpsLocation::instance()->deleteGpsFix(when);
 	populateGpsData();
 }
 
 QString QMLManager::logText() const
 {
-	QString logText = m_logText + QString("\nNumer of GPS fixes: %1").arg(locationProvider->getGpsNum());
+	QString logText = m_logText + QString("\nNumer of GPS fixes: %1").arg(GpsLocation::instance()->getGpsNum());
 	return logText;
 }
 
@@ -1659,7 +1703,7 @@ void QMLManager::appendTextToLog(const QString &newText)
 void QMLManager::setLocationServiceEnabled(bool locationServiceEnabled)
 {
 	m_locationServiceEnabled = locationServiceEnabled;
-	locationProvider->serviceEnable(m_locationServiceEnabled);
+	GpsLocation::instance()->serviceEnable(m_locationServiceEnabled);
 	emit locationServiceEnabledChanged();
 }
 
@@ -1672,7 +1716,7 @@ void QMLManager::setLocationServiceAvailable(bool locationServiceAvailable)
 
 void QMLManager::hasLocationSourceChanged()
 {
-	setLocationServiceAvailable(locationProvider->hasLocationsSource());
+	setLocationServiceAvailable(GpsLocation::instance()->hasLocationsSource());
 }
 
 void QMLManager::setVerboseEnabled(bool verboseMode)
@@ -1816,8 +1860,8 @@ QStringList QMLManager::cylinderInit() const
 		}
 	}
 
-	for (unsigned long ti = 0; ti < MAX_TANK_INFO && tank_info[ti].name != NULL; ti++) {
-		QString cyl = tank_info[ti].name;
+	for (int ti = 0; ti < tank_info_table.nr; ti++) {
+		QString cyl = tank_info_table.infos[ti].name;
 		if (cyl == "")
 			continue;
 		cylinders << cyl;
