@@ -12,6 +12,7 @@
 #include "profile-widget/diveeventitem.h"
 #include "profile-widget/divetextitem.h"
 #include "profile-widget/divetooltipitem.h"
+#include "profile-widget/divehandler.h"
 #include "core/planner.h"
 #include "core/device.h"
 #include "profile-widget/ruleritem.h"
@@ -23,7 +24,6 @@
 #include "core/divelist.h"
 #include "core/errorhelper.h"
 #ifndef SUBSURFACE_MOBILE
-#include "desktop-widgets/diveplanner.h"
 #include "desktop-widgets/simplewidgets.h"
 #include "desktop-widgets/divepicturewidget.h"
 #include "desktop-widgets/mainwindow.h"
@@ -31,7 +31,6 @@
 #include "core/qthelper.h"
 #include "core/gettextfromc.h"
 #include "core/imagedownloader.h"
-#include "core/subsurface-qt/divelistnotifier.h"
 #endif
 
 #include <libdivecomputer/parser.h>
@@ -47,16 +46,10 @@
 #ifndef QT_NO_DEBUG
 #include <QTableView>
 #endif
-#ifndef SUBSURFACE_MOBILE
-#include "desktop-widgets/preferences/preferencesdialog.h"
-#endif
 #include <QtWidgets>
 
 #define PP_GRAPHS_ENABLED (prefs.pp_graphs.po2 || prefs.pp_graphs.pn2 || prefs.pp_graphs.phe)
 
-// a couple of helpers we need
-extern bool haveFilesOnCommandLine();
-
 /* This is the global 'Item position' variable.
  * it should tell you where to position things up
  * on the canvas.
@@ -99,58 +92,64 @@ static struct _ItemPos {
 static const double thumbnailBaseZValue = 100.0;
 #endif
 
+template<typename T, class... Args>
+T *ProfileWidget2::createItem(const DiveCartesianAxis &vAxis, int vColumn, int z, Args&&... args)
+{
+	T *res = new T(*dataModel, *timeAxis, DivePlotDataModel::TIME, vAxis, vColumn,
+		       std::forward<Args>(args)...);
+	res->setZValue(static_cast<double>(z));
+	profileItems.push_back(res);
+	return res;
+}
+
 ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	currentState(INVALID),
 	dataModel(new DivePlotDataModel(this)),
 	zoomLevel(0),
 	zoomFactor(1.15),
+	isGrayscale(false),
+	printMode(false),
 	background(new DivePixmapItem()),
 	backgroundFile(":poster-icon"),
 #ifndef SUBSURFACE_MOBILE
 	toolTipItem(new ToolTipItem()),
 #endif
-	isPlotZoomed(prefs.zoomed_plot),// no! bad use of prefs. 'PreferencesDialog::loadSettings' NOT CALLED yet.
 	profileYAxis(new DepthAxis(this)),
-	gasYAxis(new PartialGasPressureAxis(this)),
+	gasYAxis(new PartialGasPressureAxis(*dataModel, this)),
 	temperatureAxis(new TemperatureAxis(this)),
 	timeAxis(new TimeAxis(this)),
-	diveProfileItem(new DiveProfileItem()),
-	temperatureItem(new DiveTemperatureItem()),
-	meanDepthItem(new DiveMeanDepthItem()),
+	diveProfileItem(createItem<DiveProfileItem>(*profileYAxis, DivePlotDataModel::DEPTH, 0)),
+	temperatureItem(createItem<DiveTemperatureItem>(*temperatureAxis, DivePlotDataModel::TEMPERATURE, 1)),
+	meanDepthItem(createItem<DiveMeanDepthItem>(*profileYAxis, DivePlotDataModel::INSTANT_MEANDEPTH, 1)),
 	cylinderPressureAxis(new DiveCartesianAxis(this)),
-	gasPressureItem(new DiveGasPressureItem()),
+	gasPressureItem(createItem<DiveGasPressureItem>(*cylinderPressureAxis, DivePlotDataModel::TEMPERATURE, 1)),
 	diveComputerText(new DiveTextItem()),
-	reportedCeiling(new DiveReportedCeiling()),
-	pn2GasItem(new PartialPressureGasItem()),
-	pheGasItem(new PartialPressureGasItem()),
-	po2GasItem(new PartialPressureGasItem()),
-	o2SetpointGasItem(new PartialPressureGasItem()),
-	ccrsensor1GasItem(new PartialPressureGasItem()),
-	ccrsensor2GasItem(new PartialPressureGasItem()),
-	ccrsensor3GasItem(new PartialPressureGasItem()),
-	ocpo2GasItem(new PartialPressureGasItem()),
-#ifndef SUBSURFACE_MOBILE
-	diveCeiling(new DiveCalculatedCeiling(this)),
+	reportedCeiling(createItem<DiveReportedCeiling>(*profileYAxis, DivePlotDataModel::CEILING, 1)),
+	pn2GasItem(createPPGas(DivePlotDataModel::PN2, PN2, PN2_ALERT, NULL, &prefs.pp_graphs.pn2_threshold)),
+	pheGasItem(createPPGas(DivePlotDataModel::PHE, PHE, PHE_ALERT, NULL, &prefs.pp_graphs.phe_threshold)),
+	po2GasItem(createPPGas(DivePlotDataModel::PO2, PO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	o2SetpointGasItem(createPPGas(DivePlotDataModel::O2SETPOINT, O2SETPOINT, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	ccrsensor1GasItem(createPPGas(DivePlotDataModel::CCRSENSOR1, CCRSENSOR1, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	ccrsensor2GasItem(createPPGas(DivePlotDataModel::CCRSENSOR2, CCRSENSOR2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	ccrsensor3GasItem(createPPGas(DivePlotDataModel::CCRSENSOR3, CCRSENSOR3, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	ocpo2GasItem(createPPGas(DivePlotDataModel::SCR_OC_PO2, SCR_OCPO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max)),
+	diveCeiling(createItem<DiveCalculatedCeiling>(*profileYAxis, DivePlotDataModel::CEILING, 1, this)),
 	decoModelParameters(new DiveTextItem()),
+#ifndef SUBSURFACE_MOBILE
 	heartBeatAxis(new DiveCartesianAxis(this)),
-	heartBeatItem(new DiveHeartrateItem()),
+	heartBeatItem(createItem<DiveHeartrateItem>(*heartBeatAxis, DivePlotDataModel::HEARTBEAT, 1)),
 	percentageAxis(new DiveCartesianAxis(this)),
-	ambPressureItem(new DiveAmbPressureItem()),
-	gflineItem(new DiveGFLineItem()),
+	ambPressureItem(createItem<DiveAmbPressureItem>(*percentageAxis, DivePlotDataModel::AMBPRESSURE, 1)),
+	gflineItem(createItem<DiveGFLineItem>(*percentageAxis, DivePlotDataModel::GFLINE, 1)),
 	mouseFollowerVertical(new DiveLineItem()),
 	mouseFollowerHorizontal(new DiveLineItem()),
 	rulerItem(new RulerItem2()),
 #endif
-	tankItem(new TankItem()),
-	isGrayscale(false),
-	printMode(false),
+	tankItem(new TankItem(*timeAxis)),
 	shouldCalculateMaxTime(true),
 	shouldCalculateMaxDepth(true),
 	fontPrintScale(1.0)
 {
-	// would like to be able to ASSERT here that PreferencesDialog::loadSettings has been called.
-	isPlotZoomed = prefs.zoomed_plot; // now it seems that 'prefs' has loaded our preferences
-
 	init_plot_info(&plotInfo);
 
 	setupSceneAndFlags();
@@ -174,6 +173,7 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	connect(&diveListNotifier, &DiveListNotifier::cylinderEdited, this, &ProfileWidget2::profileChanged);
 	connect(&diveListNotifier, &DiveListNotifier::eventsChanged, this, &ProfileWidget2::profileChanged);
 	connect(&diveListNotifier, &DiveListNotifier::pictureOffsetChanged, this, &ProfileWidget2::pictureOffsetChanged);
+	connect(&diveListNotifier, &DiveListNotifier::divesChanged, this, &ProfileWidget2::divesChanged);
 #endif // SUBSURFACE_MOBILE
 
 #if !defined(QT_NO_DEBUG) && defined(SHOW_PLOT_INFO_TABLE)
@@ -181,6 +181,29 @@ ProfileWidget2::ProfileWidget2(QWidget *parent) : QGraphicsView(parent),
 	diveDepthTableView->setModel(dataModel);
 	diveDepthTableView->show();
 #endif
+
+	auto tec = qPrefTechnicalDetails::instance();
+	connect(tec, &qPrefTechnicalDetails::calcalltissuesChanged           , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcceilingChanged              , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::gflowChanged                    , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::gfhighChanged                   , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::dcceilingChanged                , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::eadChanged                      , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcceiling3mChanged            , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::modChanged                      , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::calcndlttsChanged               , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::hrgraphChanged                  , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::rulergraphChanged               , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::show_sacChanged                 , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::zoomed_plotChanged              , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::show_pictures_in_profileChanged , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::tankbarChanged                  , this, &ProfileWidget2::actionRequestedReplot);
+	connect(tec, &qPrefTechnicalDetails::percentagegraphChanged          , this, &ProfileWidget2::actionRequestedReplot);
+
+	auto pp_gas = qPrefPartialPressureGas::instance();
+	connect(pp_gas, &qPrefPartialPressureGas::pheChanged, this, &ProfileWidget2::actionRequestedReplot);
+	connect(pp_gas, &qPrefPartialPressureGas::pn2Changed, this, &ProfileWidget2::actionRequestedReplot);
+	connect(pp_gas, &qPrefPartialPressureGas::po2Changed, this, &ProfileWidget2::actionRequestedReplot);
 }
 
 ProfileWidget2::~ProfileWidget2()
@@ -210,34 +233,19 @@ void ProfileWidget2::addItemsToScene()
 	scene()->addItem(gasYAxis);
 	scene()->addItem(temperatureAxis);
 	scene()->addItem(timeAxis);
-	scene()->addItem(diveProfileItem);
 	scene()->addItem(cylinderPressureAxis);
-	scene()->addItem(temperatureItem);
-	scene()->addItem(meanDepthItem);
-	scene()->addItem(gasPressureItem);
 	// I cannot seem to figure out if an object that I find with itemAt() on the scene
 	// is the object I am looking for - my guess is there's a simple way in Qt to do that
 	// but nothing I tried worked.
 	// so instead this adds a special magic key/value pair to the object to mark it
 	diveComputerText->setData(SUBSURFACE_OBJ_DATA, SUBSURFACE_OBJ_DC_TEXT);
 	scene()->addItem(diveComputerText);
-	scene()->addItem(reportedCeiling);
 	scene()->addItem(tankItem);
-	scene()->addItem(pn2GasItem);
-	scene()->addItem(pheGasItem);
-	scene()->addItem(po2GasItem);
-	scene()->addItem(o2SetpointGasItem);
-	scene()->addItem(ccrsensor1GasItem);
-	scene()->addItem(ccrsensor2GasItem);
-	scene()->addItem(ccrsensor3GasItem);
-	scene()->addItem(ocpo2GasItem);
+	scene()->addItem(decoModelParameters);
 #ifndef SUBSURFACE_MOBILE
 	scene()->addItem(toolTipItem);
-	scene()->addItem(diveCeiling);
-	scene()->addItem(decoModelParameters);
 	scene()->addItem(percentageAxis);
 	scene()->addItem(heartBeatAxis);
-	scene()->addItem(heartBeatItem);
 	scene()->addItem(rulerItem);
 	scene()->addItem(rulerItem->sourceNode());
 	scene()->addItem(rulerItem->destNode());
@@ -247,16 +255,9 @@ void ProfileWidget2::addItemsToScene()
 	pen.setWidth(0);
 	mouseFollowerHorizontal->setPen(pen);
 	mouseFollowerVertical->setPen(pen);
-
-	Q_FOREACH (DiveCalculatedTissue *tissue, allTissues) {
-		scene()->addItem(tissue);
-	}
-	Q_FOREACH (DivePercentageItem *percentage, allPercentages) {
-		scene()->addItem(percentage);
-	}
-	scene()->addItem(ambPressureItem);
-	scene()->addItem(gflineItem);
 #endif
+	for (AbstractProfilePolygonItem *item: profileItems)
+		scene()->addItem(item);
 }
 
 void ProfileWidget2::setupItemOnScene()
@@ -282,7 +283,6 @@ void ProfileWidget2::setupItemOnScene()
 	gasYAxis->setTickInterval(1);
 	gasYAxis->setTickSize(1);
 	gasYAxis->setMinimum(0);
-	gasYAxis->setModel(dataModel);
 	gasYAxis->setFontLabelScale(0.7);
 	gasYAxis->setLineSize(96);
 
@@ -312,45 +312,23 @@ void ProfileWidget2::setupItemOnScene()
 	diveComputerText->setAlignment(Qt::AlignRight | Qt::AlignTop);
 	diveComputerText->setBrush(getColor(TIME_TEXT, isGrayscale));
 
-	tankItem->setHorizontalAxis(timeAxis);
-
 #ifndef SUBSURFACE_MOBILE
 	rulerItem->setAxis(timeAxis, profileYAxis);
-
+#endif
 	// show the deco model parameters at the top in the center
 	decoModelParameters->setY(0);
 	decoModelParameters->setX(50);
 	decoModelParameters->setBrush(getColor(PRESSURE_TEXT));
 	decoModelParameters->setAlignment(Qt::AlignHCenter | Qt::AlignBottom);
-	setupItem(diveCeiling, profileYAxis, DivePlotDataModel::CEILING, DivePlotDataModel::TIME, 1);
+#ifndef SUBSURFACE_MOBILE
 	for (int i = 0; i < 16; i++) {
-		DiveCalculatedTissue *tissueItem = new DiveCalculatedTissue(this);
-		setupItem(tissueItem, profileYAxis, DivePlotDataModel::TISSUE_1 + i, DivePlotDataModel::TIME, 1 + i);
+		DiveCalculatedTissue *tissueItem = createItem<DiveCalculatedTissue>(*profileYAxis, DivePlotDataModel::TISSUE_1 + i, i + 1, this);
 		allTissues.append(tissueItem);
-		DivePercentageItem *percentageItem = new DivePercentageItem(i);
-		setupItem(percentageItem, percentageAxis, DivePlotDataModel::PERCENTAGE_1 + i, DivePlotDataModel::TIME, 1 + i);
+		DivePercentageItem *percentageItem = createItem<DivePercentageItem>(*percentageAxis, DivePlotDataModel::PERCENTAGE_1 + i, i + 1, i);
 		allPercentages.append(percentageItem);
 	}
-	setupItem(heartBeatItem, heartBeatAxis, DivePlotDataModel::HEARTBEAT, DivePlotDataModel::TIME, 1);
-	setupItem(ambPressureItem, percentageAxis, DivePlotDataModel::AMBPRESSURE, DivePlotDataModel::TIME, 1);
-	setupItem(gflineItem, percentageAxis, DivePlotDataModel::GFLINE, DivePlotDataModel::TIME, 1);
 #endif
-	setupItem(reportedCeiling, profileYAxis, DivePlotDataModel::CEILING, DivePlotDataModel::TIME, 1);
-	setupItem(gasPressureItem, cylinderPressureAxis, DivePlotDataModel::TEMPERATURE, DivePlotDataModel::TIME, 1);
-	setupItem(temperatureItem, temperatureAxis, DivePlotDataModel::TEMPERATURE, DivePlotDataModel::TIME, 1);
-	setupItem(diveProfileItem, profileYAxis, DivePlotDataModel::DEPTH, DivePlotDataModel::TIME, 0);
-	setupItem(meanDepthItem, profileYAxis, DivePlotDataModel::INSTANT_MEANDEPTH, DivePlotDataModel::TIME, 1);
 
-	createPPGas(pn2GasItem, DivePlotDataModel::PN2, PN2, PN2_ALERT, NULL, &prefs.pp_graphs.pn2_threshold);
-	createPPGas(pheGasItem, DivePlotDataModel::PHE, PHE, PHE_ALERT, NULL, &prefs.pp_graphs.phe_threshold);
-	createPPGas(po2GasItem, DivePlotDataModel::PO2, PO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(o2SetpointGasItem, DivePlotDataModel::O2SETPOINT, O2SETPOINT, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(ccrsensor1GasItem, DivePlotDataModel::CCRSENSOR1, CCRSENSOR1, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(ccrsensor2GasItem, DivePlotDataModel::CCRSENSOR2, CCRSENSOR2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(ccrsensor3GasItem, DivePlotDataModel::CCRSENSOR3, CCRSENSOR3, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-	createPPGas(ocpo2GasItem, DivePlotDataModel::SCR_OC_PO2, SCR_OCPO2, PO2_ALERT, &prefs.pp_graphs.po2_threshold_min, &prefs.pp_graphs.po2_threshold_max);
-
-#undef CREATE_PP_GAS
 #ifndef SUBSURFACE_MOBILE
 
 	// Visibility Connections
@@ -384,14 +362,13 @@ void ProfileWidget2::replot()
 	plotDive(current_dive, true, false);
 }
 
-void ProfileWidget2::createPPGas(PartialPressureGasItem *item, int verticalColumn, color_index_t color, color_index_t colorAlert,
-				 const double *thresholdSettingsMin, const double *thresholdSettingsMax)
+PartialPressureGasItem *ProfileWidget2::createPPGas(int column, color_index_t color, color_index_t colorAlert,
+						    const double *thresholdSettingsMin, const double *thresholdSettingsMax)
 {
-	setupItem(item, gasYAxis, verticalColumn, DivePlotDataModel::TIME, 0);
+	PartialPressureGasItem *item = createItem<PartialPressureGasItem>(*gasYAxis, column, 99);
 	item->setThresholdSettingsKey(thresholdSettingsMin, thresholdSettingsMax);
 	item->setColors(getColor(color, isGrayscale), getColor(colorAlert, isGrayscale));
-	item->settingsChanged();
-	item->setZValue(99);
+	return item;
 }
 
 void ProfileWidget2::setupItemSizes()
@@ -519,17 +496,6 @@ void ProfileWidget2::setupItemSizes()
 #endif
 }
 
-void ProfileWidget2::setupItem(AbstractProfilePolygonItem *item, DiveCartesianAxis *vAxis,
-			       int vData, int hData, int zValue)
-{
-	item->setHorizontalAxis(timeAxis);
-	item->setVerticalAxis(vAxis);
-	item->setModel(dataModel);
-	item->setVerticalDataColumn(vData);
-	item->setHorizontalDataColumn(hData);
-	item->setZValue(zValue);
-}
-
 void ProfileWidget2::setupSceneAndFlags()
 {
 	setScene(new QGraphicsScene(this));
@@ -556,11 +522,9 @@ void ProfileWidget2::resetZoom()
 // Currently just one dive, but the plan is to enable All of the selected dives.
 void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPictures, bool instant)
 {
-	static bool firstCall = true;
-#ifndef SUBSURFACE_MOBILE
 	QElapsedTimer measureDuration; // let's measure how long this takes us (maybe we'll turn of TTL calculation later
 	measureDuration.start();
-#else
+#ifdef SUBSURFACE_MOBILE
 	Q_UNUSED(doClearPictures);
 #endif
 	if (currentState != ADD && currentState != PLAN) {
@@ -578,11 +542,12 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 
 		// this copies the dive and makes copies of all the relevant additional data
 		copy_dive(d, &displayed_dive);
-#ifndef SUBSURFACE_MOBILE
-		if (decoMode() == VPMB)
+
+		if (decoMode(false) == VPMB)
 			decoModelParameters->setText(QString("VPM-B +%1").arg(prefs.vpmb_conservatism));
 		else
 			decoModelParameters->setText(QString("GF %1/%2").arg(prefs.gflow).arg(prefs.gfhigh));
+#ifndef SUBSURFACE_MOBILE
 	} else {
 		DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 		plannerModel->createTemporaryPlan();
@@ -591,19 +556,19 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 			plannerModel->deleteTemporaryPlan();
 			return;
 		}
-		if (decoMode() == VPMB)
+		if (decoMode(currentState == PLAN) == VPMB)
 			decoModelParameters->setText(QString("VPM-B +%1").arg(diveplan.vpmb_conservatism));
 		else
 			decoModelParameters->setText(QString("GF %1/%2").arg(diveplan.gflow).arg(diveplan.gfhigh));
 #endif
 	}
 
-	// special handling for the first time we display things
-	animSpeed = instant ? 0 : qPrefDisplay::animation_speed();
-	if (firstCall && haveFilesOnCommandLine()) {
-		animSpeed = 0;
-		firstCall = false;
-	}
+	struct divecomputer *currentdc = select_dc(&displayed_dive);
+	if (!currentdc || !currentdc->samples)
+		return setEmptyState();
+
+	// special handling when switching from empty state
+	animSpeed = instant || currentState == EMPTY ? 0 : qPrefDisplay::animation_speed();
 
 	// restore default zoom level
 	resetZoom();
@@ -616,14 +581,6 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 	if (currentState == EMPTY)
 		setProfileState();
 
-	// next get the dive computer structure - if there are no samples
-	// let's create a fake profile that's somewhat reasonable for the
-	// data that we have
-	struct divecomputer *currentdc = select_dc(&displayed_dive);
-	Q_ASSERT(currentdc);
-	if (!currentdc || !currentdc->samples)
-		fake_dc(currentdc);
-
 	bool setpointflag = (currentdc->divemode == CCR) && prefs.pp_graphs.po2 && current_dive;
 	bool sensorflag = setpointflag && prefs.show_ccr_sensors;
 	o2SetpointGasItem->setVisible(setpointflag && prefs.show_ccr_setpoint);
@@ -632,6 +589,7 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 	ccrsensor3GasItem->setVisible(sensorflag && (currentdc->no_o2sensors > 2));
 	ocpo2GasItem->setVisible((currentdc->divemode == PSCR) && prefs.show_scr_ocpo2);
 
+
 	/* This struct holds all the data that's about to be plotted.
 	 * I'm not sure this is the best approach ( but since we are
 	 * interpolating some points of the Dive, maybe it is... )
@@ -642,7 +600,9 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 
 	// create_plot_info_new() automatically frees old plot data
 #ifndef SUBSURFACE_MOBILE
-	create_plot_info_new(&displayed_dive, currentdc, &plotInfo, !shouldCalculateMaxDepth, &DivePlannerPointsModel::instance()->final_deco_state);
+	// A non-null planner_ds signals to create_plot_info_new that the dive is currently planned.
+	struct deco_state *planner_ds = currentState == PLAN ? &DivePlannerPointsModel::instance()->final_deco_state : nullptr;
+	create_plot_info_new(&displayed_dive, currentdc, &plotInfo, !shouldCalculateMaxDepth, planner_ds);
 #else
 	create_plot_info_new(&displayed_dive, currentdc, &plotInfo, !shouldCalculateMaxDepth, nullptr);
 #endif
@@ -663,7 +623,7 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 		maxdepth = newMaxDepth;
 	}
 
-	dataModel->setDive(&displayed_dive, plotInfo);
+	dataModel->setDive(plotInfo);
 #ifndef SUBSURFACE_MOBILE
 	toolTipItem->setPlotInfo(plotInfo);
 #endif
@@ -722,7 +682,7 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 	cylinderPressureAxis->setMinimum(plotInfo.minpressure);
 	cylinderPressureAxis->setMaximum(plotInfo.maxpressure);
 #ifndef SUBSURFACE_MOBILE
-	rulerItem->setPlotInfo(plotInfo);
+	rulerItem->setPlotInfo(&displayed_dive, plotInfo);
 #endif
 
 #ifdef SUBSURFACE_MOBILE
@@ -762,9 +722,14 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 		ocpo2GasItem->setVisible(false);
 	}
 #endif
-	tankItem->setData(dataModel, &plotInfo, &displayed_dive);
+	tankItem->setData(&plotInfo, &displayed_dive);
+
+	gasYAxis->update();
+
+	// Replot dive items
+	for (AbstractProfilePolygonItem *item: profileItems)
+		item->replot(&displayed_dive, currentState == PLAN);
 
-	dataModel->emitDataChanged();
 	// The event items are a bit special since we don't know how many events are going to
 	// exist on a dive, so I cant create cache items for that. that's why they are here
 	// while all other items are up there on the constructor.
@@ -794,7 +759,7 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 		item->setHorizontalAxis(timeAxis);
 		item->setVerticalAxis(profileYAxis, qPrefDisplay::animation_speed());
 		item->setModel(dataModel);
-		item->setEvent(event, lastgasmix);
+		item->setEvent(&displayed_dive, event, lastgasmix);
 		item->setZValue(2);
 #ifndef SUBSURFACE_MOBILE
 		item->setScale(printMode ? 4 :1);
@@ -835,22 +800,28 @@ void ProfileWidget2::plotDive(const struct dive *d, bool force, bool doClearPict
 	else
 		plotPicturesInternal(d, instant);
 
-	toolTipItem->refresh(mapToScene(mapFromGlobal(QCursor::pos())));
+	toolTipItem->refresh(&displayed_dive, mapToScene(mapFromGlobal(QCursor::pos())), currentState == PLAN);
 #endif
 
 	// OK, how long did this take us? Anything above the second is way too long,
 	// so if we are calculation TTS / NDL then let's force that off.
-#ifndef SUBSURFACE_MOBILE
-	if (measureDuration.elapsed() > 1000 && prefs.calcndltts) {
+	qint64 elapsedTime = measureDuration.elapsed();
+	if (verbose)
+		qDebug() << "Profile calculation for dive " << d->number << "took" << elapsedTime << "ms" << " -- calculated ceiling preference is" << prefs.calcceiling;
+	if (elapsedTime > 1000 && prefs.calcndltts) {
 		qPrefTechnicalDetails::set_calcndltts(false);
 		report_error(qPrintable(tr("Show NDL / TTS was disabled because of excessive processing time")));
 	}
-#endif
 }
 
-void ProfileWidget2::dateTimeChanged()
+void ProfileWidget2::divesChanged(const QVector<dive *> &dives, DiveField field)
 {
-	emit dateTimeChangedItems();
+	// If the mode of the currently displayed dive changed, replot
+	if (field.mode &&
+	    std::any_of(dives.begin(), dives.end(),
+			[id = displayed_dive.id] (const dive *d)
+			{ return d->id == id; } ))
+		replot();
 }
 
 void ProfileWidget2::actionRequestedReplot(bool)
@@ -860,18 +831,8 @@ void ProfileWidget2::actionRequestedReplot(bool)
 
 void ProfileWidget2::settingsChanged()
 {
-	// if we are showing calculated ceilings then we have to replot()
-	// because the GF could have changed; otherwise we try to avoid replot()
-	// but always replot in PLAN/ADD/EDIT mode to avoid a bug of DiveHandlers not
-	// being redrawn on setting changes, causing them to become unattached
-	// to the profile
-	bool needReplot;
-	if (currentState == ADD || currentState == PLAN || currentState ==  EDIT)
-		needReplot = true;
-	else
-		needReplot = prefs.calcceiling;
 #ifndef SUBSURFACE_MOBILE
-	gasYAxis->settingsChanged();	// Initialize ticks of partial pressure graph
+	gasYAxis->update();	// Initialize ticks of partial pressure graph
 	if ((prefs.percentagegraph||prefs.hrgraph) && PP_GRAPHS_ENABLED) {
 		profileYAxis->animateChangeLine(itemPos.depth.shrinked);
 		temperatureAxis->setPos(itemPos.temperatureAll.pos.on);
@@ -926,12 +887,7 @@ void ProfileWidget2::settingsChanged()
 	}
 
 	tankItem->setVisible(prefs.tankbar);
-	if (prefs.zoomed_plot != isPlotZoomed) {
-		isPlotZoomed = prefs.zoomed_plot;
-		needReplot = true;
-	}
-	if (needReplot)
-		replot();
+	replot();
 }
 
 void ProfileWidget2::resizeEvent(QResizeEvent *event)
@@ -1006,6 +962,16 @@ void ProfileWidget2::scale(qreal sx, qreal sy)
 #endif
 }
 
+bool ProfileWidget2::isPointOutOfBoundaries(const QPointF &point) const
+{
+	double xpos = timeAxis->valueAt(point);
+	double ypos = profileYAxis->valueAt(point);
+	return xpos > timeAxis->maximum() ||
+	       xpos < timeAxis->minimum() ||
+	       ypos > profileYAxis->maximum() ||
+	       ypos < profileYAxis->minimum();
+}
+
 #ifndef SUBSURFACE_MOBILE
 void ProfileWidget2::wheelEvent(QWheelEvent *event)
 {
@@ -1044,16 +1010,6 @@ void ProfileWidget2::mouseDoubleClickEvent(QMouseEvent *event)
 	}
 }
 
-bool ProfileWidget2::isPointOutOfBoundaries(const QPointF &point) const
-{
-	double xpos = timeAxis->valueAt(point);
-	double ypos = profileYAxis->valueAt(point);
-	return xpos > timeAxis->maximum() ||
-	       xpos < timeAxis->minimum() ||
-	       ypos > profileYAxis->maximum() ||
-	       ypos < profileYAxis->minimum();
-}
-
 void ProfileWidget2::scrollViewTo(const QPoint &pos)
 {
 	/* since we cannot use translate() directly on the scene we hack on
@@ -1071,7 +1027,7 @@ void ProfileWidget2::scrollViewTo(const QPoint &pos)
 void ProfileWidget2::mouseMoveEvent(QMouseEvent *event)
 {
 	QPointF pos = mapToScene(event->pos());
-	toolTipItem->refresh(mapToScene(mapFromGlobal(QCursor::pos())));
+	toolTipItem->refresh(&displayed_dive, mapToScene(mapFromGlobal(QCursor::pos())), currentState == PLAN);
 
 	if (zoomLevel == 0) {
 		QGraphicsView::mouseMoveEvent(event);
@@ -1144,11 +1100,11 @@ void ProfileWidget2::setEmptyState()
 	ccrsensor2GasItem->setVisible(false);
 	ccrsensor3GasItem->setVisible(false);
 	ocpo2GasItem->setVisible(false);
+	decoModelParameters->setVisible(false);
+	diveCeiling->setVisible(false);
 #ifndef SUBSURFACE_MOBILE
 	toolTipItem->clearPlotInfo();
 	toolTipItem->setVisible(false);
-	diveCeiling->setVisible(false);
-	decoModelParameters->setVisible(false);
 	rulerItem->setVisible(false);
 	ambPressureItem->setVisible(false);
 	gflineItem->setVisible(false);
@@ -1157,6 +1113,8 @@ void ProfileWidget2::setEmptyState()
 	heartBeatAxis->setVisible(false);
 	heartBeatItem->setVisible(false);
 #endif
+	for (AbstractProfilePolygonItem *item: profileItems)
+		item->clear();
 
 #ifndef SUBSURFACE_MOBILE
 	hideAll(allTissues);
@@ -1260,9 +1218,10 @@ void ProfileWidget2::setProfileState()
 	ocpo2GasItem->setVisible(current_dive && (current_dc->divemode == PSCR) && prefs.show_scr_ocpo2);
 
 	heartBeatItem->setVisible(prefs.hrgraph);
+#endif
 	diveCeiling->setVisible(prefs.calcceiling);
 	decoModelParameters->setVisible(prefs.calcceiling);
-
+#ifndef SUBSURFACE_MOBILE
 	if (prefs.calcalltissues) {
 		Q_FOREACH (DiveCalculatedTissue *tissue, allTissues) {
 			tissue->setVisible(true);
@@ -1300,17 +1259,6 @@ void ProfileWidget2::setProfileState()
 }
 
 #ifndef SUBSURFACE_MOBILE
-void ProfileWidget2::clearHandlers()
-{
-	if (handles.count()) {
-		foreach (DiveHandler *handle, handles) {
-			scene()->removeItem(handle);
-			delete handle;
-		}
-		handles.clear();
-	}
-}
-
 void ProfileWidget2::setToolTipVisibile(bool visible)
 {
 	toolTipItem->setVisible(visible);
@@ -1321,7 +1269,6 @@ void ProfileWidget2::setAddState()
 	if (currentState == ADD)
 		return;
 
-	clearHandlers();
 	setProfileState();
 	mouseFollowerHorizontal->setVisible(true);
 	mouseFollowerVertical->setVisible(true);
@@ -1339,10 +1286,8 @@ void ProfileWidget2::setAddState()
 	DivePlannerPointsModel *plannerModel = DivePlannerPointsModel::instance();
 	connect(plannerModel, &DivePlannerPointsModel::dataChanged, this, &ProfileWidget2::replot);
 	connect(plannerModel, &DivePlannerPointsModel::cylinderModelEdited, this, &ProfileWidget2::replot);
-#ifndef SUBSURFACE_MOBILE
 	connect(plannerModel, &DivePlannerPointsModel::rowsInserted, this, &ProfileWidget2::pointInserted);
 	connect(plannerModel, &DivePlannerPointsModel::rowsRemoved, this, &ProfileWidget2::pointsRemoved);
-#endif
 	/* show the same stuff that the profile shows. */
 	currentState = ADD; /* enable the add state. */
 	diveCeiling->setVisible(true);
@@ -1382,7 +1327,7 @@ void ProfileWidget2::setPlanState()
 }
 #endif
 
-bool ProfileWidget2::isPlanner()
+bool ProfileWidget2::isPlanner() const
 {
 	return currentState == PLAN;
 }
@@ -1669,7 +1614,7 @@ void ProfileWidget2::changeGas(int tank, int seconds)
 }
 #endif
 
-bool ProfileWidget2::getPrintMode()
+bool ProfileWidget2::getPrintMode() const
 {
 	return printMode;
 }
@@ -1702,7 +1647,7 @@ void ProfileWidget2::setFontPrintScale(double scale)
 	emit fontPrintScaleChanged(scale);
 }
 
-double ProfileWidget2::getFontPrintScale()
+double ProfileWidget2::getFontPrintScale() const
 {
 	if (printMode)
 		return fontPrintScale;
@@ -1751,7 +1696,7 @@ void ProfileWidget2::disconnectTemporaryConnections()
 #ifndef SUBSURFACE_MOBILE
 void ProfileWidget2::pointInserted(const QModelIndex&, int, int)
 {
-	DiveHandler *item = new DiveHandler();
+	DiveHandler *item = new DiveHandler(&displayed_dive);
 	scene()->addItem(item);
 	handles << item;
 
@@ -2072,17 +2017,19 @@ void ProfileWidget2::updateThumbnail(QString filename, QImage thumbnail, duratio
 }
 
 // Create a PictureEntry object and add its thumbnail to the scene if profile pictures are shown.
-ProfileWidget2::PictureEntry::PictureEntry(offset_t offsetIn, const QString &filenameIn, QGraphicsScene *scene, bool synchronous) : offset(offsetIn),
+ProfileWidget2::PictureEntry::PictureEntry(offset_t offsetIn, const QString &filenameIn, ProfileWidget2 *profile, bool synchronous) : offset(offsetIn),
 	duration(duration_t {0}),
 	filename(filenameIn),
 	thumbnail(new DivePictureItem)
 {
+	QGraphicsScene *scene = profile->scene();
 	int size = Thumbnailer::defaultThumbnailSize();
 	scene->addItem(thumbnail.get());
 	thumbnail->setVisible(prefs.show_pictures_in_profile);
 	QImage img = Thumbnailer::instance()->fetchThumbnail(filename, synchronous).scaled(size, size, Qt::KeepAspectRatio);
 	thumbnail->setPixmap(QPixmap::fromImage(img));
 	thumbnail->setFileUrl(filename);
+	connect(thumbnail.get(), &DivePictureItem::removePicture, profile, &ProfileWidget2::removePicture);
 }
 
 // Define a default sort order for picture-entries: sort lexicographically by timestamp and filename.
@@ -2162,7 +2109,7 @@ void ProfileWidget2::plotPicturesInternal(const struct dive *d, bool synchronous
 	// Note that FOR_EACH_PICTURE handles d being null gracefully.
 	FOR_EACH_PICTURE(d) {
 		if (picture->offset.seconds > 0 && picture->offset.seconds <= d->duration.seconds)
-			pictures.emplace_back(picture->offset, QString(picture->filename), scene(), synchronous);
+			pictures.emplace_back(picture->offset, QString(picture->filename), this, synchronous);
 	}
 	if (pictures.empty())
 		return;
@@ -2201,7 +2148,7 @@ void ProfileWidget2::picturesAdded(dive *d, QVector<PictureObj> pics)
 
 	for (const PictureObj &pic: pics) {
 		if (pic.offset.seconds > 0 && pic.offset.seconds <= d->duration.seconds) {
-			pictures.emplace_back(pic.offset, QString::fromStdString(pic.filename), scene(), false);
+			pictures.emplace_back(pic.offset, QString::fromStdString(pic.filename), this, false);
 			updateThumbnailXPos(pictures.back());
 		}
 	}
@@ -2213,6 +2160,13 @@ void ProfileWidget2::picturesAdded(dive *d, QVector<PictureObj> pics)
 	calculatePictureYPositions();
 }
 
+void ProfileWidget2::removePicture(const QString &fileUrl)
+{
+	struct dive *d = get_dive_by_uniq_id(displayed_dive.id);
+	if (d)
+		Command::removePictures({ { d, { fileUrl.toStdString() } } });
+}
+
 void ProfileWidget2::profileChanged(dive *d)
 {
 	if (!d || d->id != displayed_dive.id)
@@ -2304,7 +2258,7 @@ void ProfileWidget2::pictureOffsetChanged(dive *d, QString filename, offset_t of
 			// The parameters are passed directly to the contructor.
 			// The call returns an iterator to the new element (which might differ from
 			// the old iterator, since the buffer might have been reallocated).
-			newPos = pictures.emplace(newPos, offset, filename, scene(), false);
+			newPos = pictures.emplace(newPos, offset, filename, this, false);
 			updateThumbnailXPos(*newPos);
 			calculatePictureYPositions();
 		}
