@@ -22,6 +22,7 @@
 #include "core/color.h"
 #include "core/pictureobj.h"
 #include "core/units.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 
 class RulerItem2;
 struct dive;
@@ -78,16 +79,14 @@ public:
 	void resetZoom();
 	void scale(qreal sx, qreal sy);
 	void plotDive(const struct dive *d, bool force = false, bool clearPictures = false, bool instant = false);
-	void setupItem(AbstractProfilePolygonItem *item, DiveCartesianAxis *vAxis, int vData, int hData, int zValue);
 	void setPrintMode(bool mode, bool grayscale = false);
-	bool getPrintMode();
+	bool getPrintMode() const;
 	bool isPointOutOfBoundaries(const QPointF &point) const;
-	bool isPlanner();
-	double getFontPrintScale();
+	bool isPlanner() const;
+	double getFontPrintScale() const;
 	void setFontPrintScale(double scale);
 #ifndef SUBSURFACE_MOBILE
 	bool eventFilter(QObject *, QEvent *) override;
-	void clearHandlers();
 #endif
 	void setToolTipVisibile(bool visible);
 	State currentState;
@@ -98,15 +97,13 @@ signals:
 	void enableToolbar(bool enable);
 	void enableShortcuts();
 	void disableShortcuts(bool paste);
-	void updateDiveInfo();
 	void editCurrentDive();
-	void dateTimeChangedItems();
 
 public
 slots: // Necessary to call from QAction's signals.
-	void dateTimeChanged();
 	void settingsChanged();
 	void actionRequestedReplot(bool triggered);
+	void divesChanged(const QVector<dive *> &dives, DiveField field);
 	void setEmptyState();
 	void setProfileState();
 #ifndef SUBSURFACE_MOBILE
@@ -120,6 +117,7 @@ slots: // Necessary to call from QAction's signals.
 	void updateThumbnail(QString filename, QImage thumbnail, duration_t duration);
 	void profileChanged(dive *d);
 	void pictureOffsetChanged(dive *d, QString filename, offset_t offset);
+	void removePicture(const QString &fileUrl);
 
 	/* this is called for every move on the handlers. maybe we can speed up this a bit? */
 	void recreatePlannedDive();
@@ -155,14 +153,15 @@ private:
 	void fixBackgroundPos();
 	void scrollViewTo(const QPoint &pos);
 	void setupSceneAndFlags();
+	template<typename T, class... Args> T *createItem(const DiveCartesianAxis &vAxis, int vColumn, int z, Args&&... args);
 	void setupItemSizes();
 	void addItemsToScene();
 	void setupItemOnScene();
 	void disconnectTemporaryConnections();
 	struct plot_data *getEntryFromPos(QPointF pos);
 	void addActionShortcut(const Qt::Key shortcut, void (ProfileWidget2::*slot)());
-	void createPPGas(PartialPressureGasItem *item, int verticalColumn, color_index_t color, color_index_t colorAlert,
-			 const double *thresholdSettingsMin, const double *thresholdSettingsMax);
+	PartialPressureGasItem *createPPGas(int column, color_index_t color, color_index_t colorAlert,
+					    const double *thresholdSettingsMin, const double *thresholdSettingsMax);
 	void clearPictures();
 	void plotPicturesInternal(const struct dive *d, bool synchronous);
 	void addDivemodeSwitch(int seconds, int divemode);
@@ -180,12 +179,13 @@ private:
 	DivePlotDataModel *dataModel;
 	int zoomLevel;
 	qreal zoomFactor;
+	bool isGrayscale;
+	bool printMode;
 	DivePixmapItem *background;
 	QString backgroundFile;
 #ifndef SUBSURFACE_MOBILE
 	ToolTipItem *toolTipItem;
 #endif
-	bool isPlotZoomed;
 	// All those here should probably be merged into one structure,
 	// So it's esyer to replicate for more dives later.
 	// In the meantime, keep it here.
@@ -194,6 +194,7 @@ private:
 	PartialGasPressureAxis *gasYAxis;
 	TemperatureAxis *temperatureAxis;
 	TimeAxis *timeAxis;
+	std::vector<AbstractProfilePolygonItem *> profileItems;
 	DiveProfileItem *diveProfileItem;
 	DiveTemperatureItem *temperatureItem;
 	DiveMeanDepthItem *meanDepthItem;
@@ -210,9 +211,9 @@ private:
 	PartialPressureGasItem *ccrsensor2GasItem;
 	PartialPressureGasItem *ccrsensor3GasItem;
 	PartialPressureGasItem *ocpo2GasItem;
-#ifndef SUBSURFACE_MOBILE
 	DiveCalculatedCeiling *diveCeiling;
 	DiveTextItem *decoModelParameters;
+#ifndef SUBSURFACE_MOBILE
 	QList<DiveCalculatedTissue *> allTissues;
 	DiveCartesianAxis *heartBeatAxis;
 	DiveHeartrateItem *heartBeatItem;
@@ -225,8 +226,6 @@ private:
 	RulerItem2 *rulerItem;
 #endif
 	TankItem *tankItem;
-	bool isGrayscale;
-	bool printMode;
 
 	QList<QGraphicsSimpleTextItem *> gases;
 
@@ -242,7 +241,7 @@ private:
 		std::unique_ptr<DivePictureItem> thumbnail;
 		// For videos with known duration, we represent the duration of the video by a line
 		std::unique_ptr<QGraphicsRectItem> durationLine;
-		PictureEntry (offset_t offsetIn, const QString &filenameIn, QGraphicsScene *scene, bool synchronous);
+		PictureEntry (offset_t offsetIn, const QString &filenameIn, ProfileWidget2 *profile, bool synchronous);
 		bool operator< (const PictureEntry &e) const;
 	};
 	void updateThumbnailXPos(PictureEntry &e);
@@ -250,12 +249,14 @@ private:
 	void calculatePictureYPositions();
 	void updateDurationLine(PictureEntry &e);
 	void updateThumbnailPaintOrder();
+#endif
 
 	QList<DiveHandler *> handles;
+#ifndef SUBSURFACE_MOBILE
 	void repositionDiveHandlers();
 	int fixHandlerIndex(DiveHandler *activeHandler);
-	friend class DiveHandler;
 #endif
+	friend class DiveHandler;
 	QHash<Qt::Key, QAction *> actionsForKeys;
 	bool shouldCalculateMaxTime;
 	bool shouldCalculateMaxDepth;
