@@ -3,6 +3,7 @@
 #define QMLPROFILE_H
 
 #include "profilewidget2.h"
+#include "core/subsurface-qt/divelistnotifier.h"
 #include <QQuickPaintedItem>
 
 class QMLProfile : public QQuickPaintedItem
@@ -38,6 +39,9 @@ private:
 	QScopedPointer<ProfileWidget2> m_profileWidget;
 	void updateProfile();
 
+private slots:
+	void divesChanged(const QVector<dive *> &dives, DiveField);
+
 signals:
 	void rightAlignedChanged();
 	void devicePixelRatioChanged();
