@@ -6,7 +6,6 @@
 #include <string.h>
 #include <time.h>
 
-#include "core/color.h"
 #include "core/downloadfromdcthread.h" // for fill_computer_list
 #include "core/errorhelper.h"
 #include "core/parse.h"
@@ -15,11 +14,7 @@
 #include "core/subsurfacestartup.h"
 #include "core/settings/qPref.h"
 #include "core/tag.h"
-#include "desktop-widgets/diveplanner.h"
 #include "desktop-widgets/mainwindow.h"
-#include "desktop-widgets/preferences/preferencesdialog.h"
-#include "desktop-widgets/tab-widgets/maintab.h"
-#include "profile-widget/profilewidget2.h"
 
 #include <QApplication>
 #include <QLoggingCategory>
@@ -82,6 +77,7 @@ int main(int argc, char **argv)
 	setup_system_prefs();
 	copy_prefs(&default_prefs, &prefs);
 	fill_computer_list();
+	reset_tank_info_table(&tank_info_table);
 	parse_xml_init();
 	taglist_init_global();
 	init_ui();
@@ -120,14 +116,10 @@ int main(int argc, char **argv)
 	qPref::sync();
 
 	free_prefs();
+	clear_tank_info_table(&tank_info_table);
 	return 0;
 }
 
-bool haveFilesOnCommandLine()
-{
-	return filesOnCommandLine;
-}
-
 #define VALIDATE_GL_PREFIX "validateGL(): "
 
 void validateGL()
