@@ -1,5 +1,6 @@
 // SPDX-License-Identifier: GPL-2.0
 #include "testpicture.h"
+#include "core/device.h"
 #include "core/dive.h"
 #include "core/divesite.h"
 #include "core/errorhelper.h"
@@ -26,7 +27,7 @@ void TestPicture::addPicture()
 	struct picture *pic1, *pic2;
 	verbose = 1;
 
-	QCOMPARE(parse_file(SUBSURFACE_TEST_DATA "/dives/test44.xml", &dive_table, &trip_table, &dive_site_table), 0);
+	QCOMPARE(parse_file(SUBSURFACE_TEST_DATA "/dives/test44.xml", &dive_table, &trip_table, &dive_site_table, &device_table, &filter_preset_table), 0);
 	dive = get_dive(0);
 	// Pictures will be added to selected dives
 	dive->selected = true;
